@@ -7,7 +7,7 @@ $block_name = _("Metar Weather");
  * screen to display METAR weather data for a specified location
  * (currently airports).
  *
- * $Horde: horde/lib/Block/metar.php,v 1.22.10.3 2005/10/13 08:33:20 selsky Exp $
+ * $Horde: horde/lib/Block/metar.php,v 1.22.10.6 2006/06/23 19:31:28 jan Exp $
  *
  * @package Horde_Block
  */
@@ -47,7 +47,14 @@ class Horde_Block_Horde_metar extends Horde_Block {
                 return $db;
             }
 
-            $db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            // Set DB portability options.
+            switch ($db->phptype) {
+            case 'mssql':
+                $db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+                break;
+            default:
+                $db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
+            }
 
             $result = $db->query('SELECT icao, name, country FROM metarAirports ORDER BY country');
             if (is_a($result, 'PEAR_Error')) {
@@ -91,12 +98,12 @@ class Horde_Block_Horde_metar extends Horde_Block {
 
     function _row($label, $content)
     {
-        return '<br /><b>' . $label . ':</b> ' . $content;
+        return '<br /><strong>' . $label . ':</strong> ' . $content;
     }
 
     function _sameRow($label, $content)
     {
-        return ' <b>' . $label . ':</b> ' . $content;
+        return ' <strong>' . $label . ':</strong> ' . $content;
     }
 
     /**
@@ -129,12 +136,7 @@ class Horde_Block_Horde_metar extends Horde_Block {
 
         require_once 'Services/Weather.php';
         $metar = &Services_Weather::service('METAR', array('debug' => 0));
-        $dbString = $conf['sql']['phptype'] . '://';
-        $dbString .= $conf['sql']['username'] . ':';
-        $dbString .= $conf['sql']['password'] . '@';
-        $dbString .= $conf['sql']['hostspec'] . '/';
-        $dbString .= $conf['sql']['database'];
-        $metar->setMetarDB($dbString);
+        $metar->setMetarDB($conf['sql']);
         $metar->setUnitsFormat($this->_params['units']);
         $metar->setDateTimeFormat('M j, Y', 'H:i');
         $metar->setMetarSource('http');
@@ -142,18 +144,18 @@ class Horde_Block_Horde_metar extends Horde_Block {
         $units = $metar->getUnitsFormat($this->_params['units']);
         $weather = $metar->getWeather($this->_params['location']);
 
-        $html = '<table width="100%" border="0" cellpadding="0" cellspacing="0">' .
-            '<tr><td class="control"><b>' .
+        $html = '<table width="100%" cellspacing="0">' .
+            '<tr><td class="control"><strong>' .
             sprintf('%s, %s (%s)',
                     $metarLocs['location']['values'][$this->_params['__location']][$this->_params['location']],
                     $this->_params['__location'],
                     $this->_params['location']) .
-            '</b></td></tr></table><b>' . _("Last Updated:") . '</b> ' .
+            '</strong></td></tr></table><strong>' . _("Last Updated:") . '</strong> ' .
             $weather['update'] . '<br /><br />';
 
         // Wind.
         if (isset($weather['wind'])) {
-            $html .= '<b>' . _("Wind:") . '</b> ';
+            $html .= '<strong>' . _("Wind:") . '</strong> ';
             if ($weather['windDirection'] == 'Variable') {
                 if (!empty($this->_params['knots'])) {
                     $html .= sprintf(_("%s at %s %s"),
@@ -330,15 +332,15 @@ class Horde_Block_Horde_metar extends Horde_Block {
         if (!empty($this->_params['taf'])) {
             $taf = $metar->getForecast($this->_params['location']);
             if (!is_a($taf, 'PEAR_Error')) {
-                $forecast = '<table width="100%" border="0" cellpadding="0" cellspacing="0">';
-                $forecast .= '<tr><td class="control" colspan="2"><center><b>' . _("Forecast (TAF)") . '</b></td></tr></table>';
-                $forecast .= '<b>Valid: </b>' . $taf['validFrom'] . ' - ' . $taf['validTo'] . '<br /><br />';
+                $forecast = '<table width="100%" cellspacing="0">';
+                $forecast .= '<tr><td class="control" colspan="2"><center><strong>' . _("Forecast (TAF)") . '</strong></td></tr></table>';
+                $forecast .= '<strong>Valid: </strong>' . $taf['validFrom'] . ' - ' . $taf['validTo'] . '<br /><br />';
                 $item = 0;
                 foreach ($taf['time'] as $time => $entry) {
                     $item++;
-                    $forecast .= '<table width="100%" border="0" cellpadding="0" cellspacing="0">';
+                    $forecast .= '<table width="100%" cellspacing="0">';
                     $forecast .= '<tr class="item' . ($item % 2) . '">';
-                    $forecast .= '<td align="center" width="50">' . $time . '</td><td><b>Wind:</b> ';
+                    $forecast .= '<td align="center" width="50">' . $time . '</td><td><strong>Wind:</strong> ';
                     if (isset($entry['wind'])) {
                         if ($entry['windDirection'] == 'Variable') {
                             if (!empty($this->_params['knots'])) {
@@ -370,19 +372,19 @@ class Horde_Block_Horde_metar extends Horde_Block {
                         $forecast .= '<br />';
                     }
                     if (isset($entry['temperatureLow']) || isset($entry['temperatureHigh'])) {
-                        $forecast .= '<b>Temperature</b>';
+                        $forecast .= '<strong>Temperature</strong>';
                         if (isset($entry['temperatureLow'])) {
-                            $forecast .= '<b> Low:</b>';
+                            $forecast .= '<strong> Low:</strong>';
                             $forecast .= $entry['temperatureLow'];
                         }
                         if (isset($entry['temperatureHigh'])) {
-                            $forecast .= '<b> High:</b>';
+                            $forecast .= '<strong> High:</strong>';
                             $forecast .= $entry['temperatureHigh'];
                         }
                         $forecast .= '<br />';
                     }
                     if (isset($entry['windshear'])) {
-                        $forecast .= '<b>Windshear:</b>';
+                        $forecast .= '<strong>Windshear:</strong>';
                         $forecast .= sprintf(_("from the %s (%s) at %s %s"),
                                         $entry['windshearDirection'],
                                         $entry['windshearDegrees'],
@@ -391,16 +393,16 @@ class Horde_Block_Horde_metar extends Horde_Block {
                         $forecast .= '<br />';
                     }
                     if (isset($entry['visibility'])) {
-                        $forecast .= '<b>Visibility:</b> ';
+                        $forecast .= '<strong>Visibility:</strong> ';
                         $forecast .= strtolower($entry['visQualifier']) . ' ' . $entry['visibility'] . ' ' . $units['vis'];
                         $forecast .= '<br />';
                     }
                     if (isset($entry['condition'])) {
-                        $forecast .= '<b>Conditions:</b> ';
+                        $forecast .= '<strong>Conditions:</strong> ';
                         $forecast .= $entry['condition'];
                         $forecast .= '<br />';
                     }
-                    $forecast .= '<b>Clouds:</b> ';
+                    $forecast .= '<strong>Clouds:</strong> ';
                     foreach ($entry['clouds'] as $clouds) {
                         if (isset($clouds['type'])) {
                             $forecast .= ' ' . $clouds['type'];
@@ -420,15 +422,15 @@ class Horde_Block_Horde_metar extends Horde_Block {
                             $forecast .= '<td align="center" width="50">';
                             $forecast .= '* ' . $fmcEntry['from'] . '<br /> - ' . $fmcEntry['to'] . '</td>';
                             $forecast .= '<td>';
-                            $forecast .= '<b>Type: </b>' . $fmcEntry['type'];
+                            $forecast .= '<strong>Type: </strong>' . $fmcEntry['type'];
                             if (isset($fmcEntry['probability'])) {
-                                $forecast .= ' <b> Prob: </b>' . $fmcEntry['probability'] . '%';
+                                $forecast .= ' <strong> Prob: </strong>' . $fmcEntry['probability'] . '%';
                             }
                             if (isset($fmcEntry['condition'])) {
-                                $forecast .= ' <b> Conditions: </b>' . $fmcEntry['condition'];
+                                $forecast .= ' <strong> Conditions: </strong>' . $fmcEntry['condition'];
                             }
                             if (isset($fmcEntry['clouds'])) {
-                                $forecast .= ' <b>Clouds:</b>';
+                                $forecast .= ' <strong>Clouds:</strong>';
                                 foreach ($fmcEntry['clouds'] as $fmcClouds) {
                                     if (isset($fmcClouds['type'])) {
                                         $forecast .= ' ' . $fmcClouds['type'];
@@ -443,7 +445,7 @@ class Horde_Block_Horde_metar extends Horde_Block {
                                 }
                             }
                             if (isset($fmcEntry['visQualifier'])) {
-                                $forecast .= ' <b>Visibility:</b> ';
+                                $forecast .= ' <strong>Visibility:</strong> ';
                                 $forecast .= strtolower($fmcEntry['visQualifier']) . ' ';
                                 $forecast .= $fmcEntry['visibility'] . ' ' . $units['vis'];
                             }
