@@ -7,10 +7,10 @@ include_once 'Horde/Util.php';
  * The Horde:: class provides the functionality shared by all Horde
  * applications.
  *
- * $Horde: framework/Horde/Horde.php,v 1.489.2.26 2005/08/24 14:04:36 selsky Exp $
+ * $Horde: framework/Horde/Horde.php,v 1.489.2.66 2007/09/29 15:49:33 jan Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -87,6 +87,8 @@ class Horde {
         return true;
     }
 
+    /**
+     */
     function &getLogger()
     {
         global $conf;
@@ -122,7 +124,7 @@ class Horde {
         // Make sure to force a completely new session ID and clear
         // all session data.
         if (version_compare(phpversion(), '4.3.3') !== -1) {
-            session_regenerate_id();
+            session_regenerate_id(true);
             session_unset();
         } else {
             @session_destroy();
@@ -138,6 +140,15 @@ class Horde {
             Horde::setupSessionHandler();
             @session_start();
         }
+
+        /* Reset cookie timeouts, if necessary. */
+        if (!empty($GLOBALS['conf']['session']['timeout'])) {
+            $app = $GLOBALS['registry']->getApp();
+            if (Secret::clearKey($app)) {
+                Secret::setKey($app);
+            }
+            Secret::setKey('auth');
+        }
     }
 
     /**
@@ -147,7 +158,7 @@ class Horde {
      *                       error message.
      * @param integer $file  The file in which the error occured.
      * @param integer $line  The line on which the error occured.
-     * @param boolean $log   Log this message via Horde::logMesage()?
+     * @param boolean $log   Log this message via Horde::logMessage()?
      */
     function fatal($error, $file, $line, $log = true)
     {
@@ -186,7 +197,7 @@ class Horde {
         } elseif (is_object($error) && method_exists($error, 'getMessage')) {
             $errortext .= '<h3>' . htmlspecialchars($error->getMessage()) . '</h3>';
         } elseif (is_string($error)) {
-            $errortext .= '<h3>' . $error . '</h3>';
+            $errortext .= '<h3>' . htmlspecialchars($error) . '</h3>';
         }
 
         if ($admin) {
@@ -210,7 +221,7 @@ class Horde {
             echo <<< HTML
 <html>
 <head><title>Horde :: Fatal Error</title></head>
-<body style="background-color: white; color: black;">$errortext</body>
+<body style="background:#fff; color:#000">$errortext</body>
 </html>
 HTML;
         }
@@ -236,12 +247,25 @@ HTML;
             $app = $registry->getApp();
         }
 
+        // Skip any js files that have since been deprecated.
+        $ignored_files = array('horde' => array('tooltip.js' => 1));
+        if (!empty($ignored_files[$app][$file])) {
+            return;
+        }
+
         // Don't include scripts multiple times.
         if (!empty($included[$app][$file])) {
             return;
         }
         $included[$app][$file] = true;
 
+        // Explicitly check for a non-PHP version of the script.
+        if (!$direct && file_exists($file{0} == '/' ? $registry->get('fileroot', $app) . $file : $registry->get('jsfs', $app) . '/' . $file)) {
+            $direct = true;
+        }
+
+        // If headers have already been sent, we need to output a
+        // <script> tag directly.
         if (ob_get_length() || headers_sent()) {
             if ($direct) {
                 $url = Horde::url($file{0} == '/' ? $registry->get('webroot', $app) . $file : $registry->get('jsuri', $app) . '/' . $file);
@@ -250,7 +274,7 @@ HTML;
                 $url = Util::addParameter($url, array('file' => $file,
                                                       'app'  => $app));
             }
-            echo '<script language="JavaScript" type="text/javascript" src="' . $url . '"></script>';
+            echo '<script type="text/javascript" src="' . $url . '"></script>' . "\n";
         } else {
             global $_horde_script_files;
             $_horde_script_files[$app][] = array($file, $direct);
@@ -264,17 +288,31 @@ HTML;
     {
         global $_horde_script_files, $registry;
 
+        /* If there is no javascript available, there's no point in including
+         * the rest of the files. */
+        if (!$GLOBALS['browser']->hasFeature('javascript')) {
+            return;
+        }
+
+        /* Add general UI js library. */
+        Horde::addScriptFile('horde.js', 'horde', true);
+        if ($GLOBALS['browser']->hasQuirk('windowed_controls')) {
+            /* Fixes for IE that can't easily be done without browser
+             * detection. */
+            Horde::addScriptFile('horde.ie.js', 'horde', true);
+        }
+
         if (!empty($_horde_script_files)) {
             $base_url = Horde::url($registry->get('webroot', 'horde') . '/services/javascript.php');
             foreach ($_horde_script_files as $app => $files) {
                 foreach ($files as $file) {
                     if (!empty($file[1])) {
                         $url = $file[0]{0} == '/' ? $registry->get('webroot', $app) . $file[0] : $registry->get('jsuri', $app) . '/' . $file[0];
-                        echo '<script language="JavaScript" type="text/javascript" src="' . Horde::url($url) . "\"></script>\n";
+                        echo '<script type="text/javascript" src="' . Horde::url($url) . "\"></script>\n";
                     } else {
                         $url = Util::addParameter($base_url, array('file' => $file[0],
                                                                    'app'  => $app));
-                        echo '<script language="JavaScript" type="text/javascript" src="' . $url . "\"></script>\n";
+                        echo '<script type="text/javascript" src="' . $url . "\"></script>\n";
                     }
                 }
             }
@@ -322,14 +360,10 @@ HTML;
         switch ($type) {
         case 'help':
             if ($GLOBALS['browser']->hasFeature('javascript')) {
-                Horde::addScriptFile('open_help_win.js', 'horde');
-                return "javascript:open_help_win('$app');";
-            } else {
-                $url = Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/help/', true);
-                return Util::addParameter($url, array('module' => $app,
-                                                      'show' => 'topics'));
+                Horde::addScriptFile('popup.js', 'horde', true);
             }
-            break;
+            $url = Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/help/', true);
+            return Util::addParameter($url, 'module', $app);
 
         case 'problem':
             return Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/problem.php?return_url=' . urlencode(Horde::selfUrl(true, true, true)));
@@ -556,9 +590,10 @@ HTML;
             $url = $uri;
         }
 
-        if (($append_session == 1) ||
-            (($append_session == 0) &&
-             !isset($_COOKIE[session_name()]))) {
+        if (empty($GLOBALS['conf']['session']['use_only_cookies']) &&
+            (($append_session == 1) ||
+             (($append_session == 0) &&
+              !isset($_COOKIE[session_name()])))) {
             $url = Util::addParameter($url, session_name(), session_id());
         }
 
@@ -589,41 +624,50 @@ HTML;
      */
     function applicationUrl($uri, $full = false, $append_session = 0)
     {
-        global $registry;
-
-        /* Store the webroot in a local variable. */
-        $webroot = $registry->get('webroot');
-
         if ($full) {
             return Horde::url($uri, $full, $append_session);
-        } elseif (substr($webroot, -1) == '/') {
-            return Horde::url($webroot . $uri, $full, $append_session);
-        } else {
-            return Horde::url($webroot . '/' . $uri, $full, $append_session);
         }
+
+        if (substr($uri, 0, 1) != '/') {
+            $webroot = $GLOBALS['registry']->get('webroot');
+            if (substr($webroot, -1) != '/') {
+                $webroot .= '/';
+            }
+            $uri = $webroot . $uri;
+        }
+
+        return Horde::url($uri, $full, $append_session);
     }
 
     /**
-     * Returns an external link passed through the dereferer to strip session
-     * IDs from the referer.
+     * Returns an external link passed through the dereferrer to strip session
+     * IDs from the referrer.
      *
      * @param string $url   The external URL to link to.
      * @param boolean $tag  If true, a complete <a> tag is returned, only the
      *                      url otherwise.
      *
-     * @return string  The correct link to the dereferer script.
+     * @return string  The link to the dereferrer script.
      */
     function externalUrl($url, $tag = false)
     {
-        $ext = Horde::url($GLOBALS['registry']->get('webroot', 'horde') .
-                          '/services/go.php', true, -1);
+        if (isset($_COOKIE[session_name()]) ||
+            String::substr($url, 0, 1) == '#' ||
+            String::substr($url, 0, 7) == 'mailto:') {
+            $ext = $url;
+        } else {
+            $ext = Horde::url($GLOBALS['registry']->get('webroot', 'horde') .
+                              '/services/go.php', true, -1);
+
+            /* We must make sure there are no &amp's in the URL. */
+            $url = preg_replace(array('/(=?.*?)&amp;(.*?=)/', '/(=?.*?)&amp;(.*?=)/'), '$1&$2', $url);
+            $ext = Util::addParameter($ext, 'url', $url);
+        }
 
-        /* We must make sure there are no &amp's in the URL. */
-        $url = preg_replace(array('/(=?.*?)&amp;(.*?=)/', '/(=?.*?)&amp;(.*?=)/'), '$1&$2', $url);
-        $ext = Util::addParameter($ext, 'url', $url);
         if ($tag) {
             $ext = Horde::link($ext, $url, '', '_blank');
         }
+
         return $ext;
     }
 
@@ -705,33 +749,38 @@ HTML;
     /**
      * Returns an anchor tag with the relevant parameters
      *
-     * @param string $url        The full URL to be linked to
-     * @param string $status     The JavaScript mouse-over string
-     * @param string $class      The CSS class of the link
+     * @param string $url        The full URL to be linked to.
+     * @param string $title      The link title/description.
+     * @param string $class      The CSS class of the link.
      * @param string $target     The window target to point to.
      * @param string $onclick    JavaScript action for the 'onclick' event.
-     * @param string $title      The link title (tooltip).
+     * @param string $title2     The link title (tooltip) (deprecated - just
+     *                           use $title).
      * @param string $accesskey  The access key to use.
      * @param array $attributes  Any other name/value pairs to add to the <a>
      *                           tag.
+     * @param boolean $escape    Whether to escape special characters in the
+     *                           title attribute.
      *
      * @return string  The full <a href> tag.
      */
-    function link($url, $status = '', $class = '', $target = '', $onclick = '',
-                  $title = '', $accesskey = '', $attributes = array())
+    function link($url, $title = '', $class = '', $target = '', $onclick = '',
+                  $title2 = '', $accesskey = '', $attributes = array(),
+                  $escape = true)
     {
         static $charset;
         if (!isset($charset)) {
             $charset = NLS::getCharset();
         }
 
+        if (!empty($title2)) {
+            $title = $title2;
+        }
+
         $ret = "<a href=\"$url\"";
         if (!empty($onclick)) {
             $ret .= " onclick=\"$onclick\"";
         }
-        if (!empty($status)) {
-            $ret .= ' onmouseout="window.status=\'\';" onmouseover="window.status=\'' . @htmlspecialchars(strtr(addslashes($status), array("\r" => '', "\n" => '')), ENT_QUOTES, $charset) . '\'; return true;"';
-        }
         if (!empty($class)) {
             $ret .= " class=\"$class\"";
         }
@@ -739,7 +788,10 @@ HTML;
             $ret .= " target=\"$target\"";
         }
         if (!empty($title)) {
-            $ret .= ' title="' . @htmlspecialchars($title, ENT_QUOTES, $charset) . '"';
+            if ($escape) {
+                $title = nl2br(@htmlspecialchars(@htmlspecialchars($title, ENT_QUOTES, $charset), ENT_QUOTES, $charset));
+            }
+            $ret .= ' title="' . $title . '"';
         }
         if (!empty($accesskey)) {
             $ret .= ' accesskey="' . htmlspecialchars($accesskey) . '"';
@@ -752,40 +804,78 @@ HTML;
         return "$ret>";
     }
 
+    /**
+     * Uses DOM Tooltips to display the 'title' attribute for
+     * Horde::link() calls.
+     *
+     * @param string $url        The full URL to be linked to
+     * @param string $status     The JavaScript mouse-over string
+     * @param string $class      The CSS class of the link
+     * @param string $target     The window target to point to.
+     * @param string $onclick    JavaScript action for the 'onclick' event.
+     * @param string $title      The link title (tooltip).
+     * @param string $accesskey  The access key to use.
+     * @param array  $attributes Any other name/value pairs to add to the <a>
+     *                           tag.
+     *
+     * @return string  The full <a href> tag.
+     */
+    function linkTooltip($url, $status = '', $class = '', $target = '',
+                         $onclick = '', $title = '', $accesskey = '',
+                         $attributes = array())
+    {
+        static $charset;
+        if (!isset($charset)) {
+            $charset = NLS::getCharset();
+        }
+
+        if (!empty($title)) {
+            $title = '&lt;pre&gt;' . preg_replace(array('/\n/', '/((?<!<br)\s{1,}(?<!\/>))/em', '/<br \/><br \/>/', '/<br \/>/'), array('', 'str_repeat("&nbsp;", strlen("$1"))', '&lt;br /&gt; &lt;br /&gt;', '&lt;br /&gt;'), nl2br(@htmlspecialchars(@htmlspecialchars($title, ENT_QUOTES, $charset), ENT_QUOTES, $charset))) . '&lt;/pre&gt;';
+        }
+        return Horde::link($url, $title, $class, $target, $onclick, null, $accesskey, $attributes, false);
+    }
+
     /**
      * Returns an anchor sequence with the relevant parameters for a widget
      * with accesskey and text.
      *
-     * @param string $url       The full URL to be linked to
-     * @param string $status    The JavaScript mouse-over string
-     * @param string $class     The CSS class of the link
-     * @param string $target    The window target to point to.
-     * @param string $onclick   JavaScript action for the 'onclick' event.
-     * @param string $title     The link title (tooltip).
+     * @access public
+     *
+     * @param string  $url      The full URL to be linked to.
+     * @param string  $title    The link title/description.
+     * @param string  $class    The CSS class of the link
+     * @param string  $target   The window target to point to.
+     * @param string  $onclick  JavaScript action for the 'onclick' event.
+     * @param string  $title2   The link title (tooltip) (deprecated - just use
+     *                          $title).
      * @param boolean $nocheck  Don't check if the access key already has been
-     *                          used?
+     *                          used. Defaults to false (= check).
      *
      * @return string  The full <a href>Title</a> sequence.
      */
-    function widget($url, $status = '', $class = 'widget', $target = '',
-                    $onclick = '', $title = '', $nocheck = false)
+    function widget($url, $title = '', $class = 'widget', $target = '',
+                    $onclick = '', $title2 = '', $nocheck = false)
     {
-        $ak = Horde::getAccessKey($title, $nocheck);
-        $plaintitle = Horde::stripAccessKey($title);
+        if (!empty($title2)) {
+            $title = $title2;
+        }
 
-        return Horde::link($url, $status, $class, $target, $onclick, $plaintitle, $ak) . Horde::highlightAccessKey($title, $ak) . '</a>';
+        $ak = Horde::getAccessKey($title, $nocheck);
+
+        return Horde::link($url, '', $class, $target, $onclick, '', $ak) . Horde::highlightAccessKey($title, $ak) . '</a>';
     }
 
     /**
      * Returns a session-id-ified version of $SCRIPT_NAME resp. $PHP_SELF.
      *
-     * @param boolean $query_string  Include any QUERY_STRING?
+     * @param boolean $script_params Include script parameters like
+     *                               QUERY_STRING and PATH_INFO?
      * @param boolean $nocache       Include a nocache parameter in the URL?
      * @param boolean $full          Return a full URL?
      *
      * @return string  The requested URI.
      */
-    function selfUrl($query_string = false, $nocache = true, $full = false,
+    function selfUrl($script_params = false, $nocache = true, $full = false,
                      $force_ssl = false)
     {
         if (substr(php_sapi_name(), 0, 3) == 'cgi') {
@@ -799,14 +889,19 @@ HTML;
                 $_SERVER['PHP_SELF'];
         }
 
-        if ($query_string && !empty($_SERVER['QUERY_STRING'])) {
-            $url .= '?' . $_SERVER['QUERY_STRING'];
+        if ($script_params) {
+            if (!empty($_SERVER['PATH_INFO'])) {
+                $url .= $_SERVER['PATH_INFO'];
+            }
+            if (!empty($_SERVER['QUERY_STRING'])) {
+                $url .= '?' . $_SERVER['QUERY_STRING'];
+            }
         }
 
         $url = Horde::url($url, $full, 0, $force_ssl);
 
         if ($nocache) {
-            return Util::nocacheUrl($url);
+            return Util::nocacheUrl($url, !$full);
         } else {
             return $url;
         }
@@ -849,11 +944,13 @@ HTML;
 
         /* Build all of the tag attributes. */
         $attributes = array('src' => $src,
-                            'alt' => $alt,
-                            'title' => $alt);
+                            'alt' => $alt);
         if (is_array($attr)) {
             $attributes = array_merge($attributes, $attr);
         }
+        if (empty($attributes['title'])) {
+            $attributes['title'] = '';
+        }
 
         $img = '<img';
         foreach ($attributes as $attribute => $value) {
@@ -873,7 +970,7 @@ HTML;
      * Determines the location of the system temporary directory. If a specific
      * setting cannot be found, it defaults to /tmp.
      *
-     * @return string  A directory name which can be used for temp files.
+     * @return string  A directory name that can be used for temp files.
      *                 Returns false if one could not be found.
      */
     function getTempDir()
@@ -921,8 +1018,6 @@ HTML;
 
     /**
      * Starts output compression, if requested.
-     *
-     * @since Horde 2.2
      */
     function compressOutput()
     {
@@ -935,8 +1030,8 @@ HTML;
         /* Compress output if requested and possible. */
         if ($GLOBALS['conf']['compress_pages'] &&
             !$GLOBALS['browser']->hasQuirk('buggy_compression') &&
-            ini_get('zlib.output_compression') == '' &&
-            ini_get('zend_accelerator.compress_all') == '' &&
+            !(bool)ini_get('zlib.output_compression') &&
+            !(bool)ini_get('zend_accelerator.compress_all') &&
             ini_get('output_handler') != 'ob_gzhandler') {
             if (ob_get_level()) {
                 ob_end_clean();
@@ -1019,11 +1114,13 @@ HTML;
      */
     function stylesheetLink($apps = null, $theme = '', $inherit = true)
     {
-        if ($theme !== false && empty($theme)) {
+        if ($theme === '' && isset($GLOBALS['prefs'])) {
             $theme = $GLOBALS['prefs']->getValue('theme');
         }
 
+        $rtl = isset($GLOBALS['nls']['rtl'][$GLOBALS['language']]);
         $css = array();
+
         $themes_fs = $GLOBALS['registry']->get('themesfs', 'horde');
         if ($inherit) {
             $themes_uri = Horde::url($GLOBALS['registry']->get('themesuri', 'horde'), false, -1);
@@ -1032,26 +1129,45 @@ HTML;
                 file_exists($themes_fs . '/' . $theme . '/screen.css')) {
                 $css[] = $themes_uri . '/' . $theme . '/screen.css';
             }
+
+            if ($rtl) {
+                $css[] = $themes_uri . '/rtl.css';
+                if (!empty($theme) &&
+                    file_exists($themes_fs . '/' . $theme . '/rtl.css')) {
+                    $css[] = $themes_uri . '/' . $theme . '/rtl.css';
+                }
+            }
         }
 
         if (!empty($apps)) {
             if (!is_array($apps)) {
                 $apps = array($apps);
             }
+
             foreach ($apps as $app) {
                 if ($inherit && $app == 'horde') {
                     continue;
                 }
 
                 $themes_fs = $GLOBALS['registry']->get('themesfs', $app);
+                $themes_uri = Horde::url($GLOBALS['registry']->get('themesuri', $app), false, -1);
                 if (file_exists($themes_fs . '/screen.css')) {
-                    $themes_uri = Horde::url($GLOBALS['registry']->get('themesuri', $app), false, -1);
                     $css[] = $themes_uri . '/screen.css';
                 }
                 if (!empty($theme) &&
                     file_exists($themes_fs . '/' . $theme . '/screen.css')) {
                     $css[] = $themes_uri . '/' . $theme . '/screen.css';
                 }
+
+                if ($rtl) {
+                    if (file_exists($themes_fs . '/rtl.css')) {
+                        $css[] = $themes_uri . '/rtl.css';
+                    }
+                    if (!empty($theme) &&
+                        file_exists($themes_fs . '/' . $theme . '/rtl.css')) {
+                        $css[] = $themes_uri . '/' . $theme . '/rtl.css';
+                    }
+                }
             }
         }
 
@@ -1068,8 +1184,8 @@ HTML;
         }
 
         /* Load browser specific stylesheets if needed. */
-        if ($GLOBALS['browser']->isBrowser('msie')) {
-            $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/ie.css" rel="stylesheet" type="text/css" />' . "\n";
+        if ($GLOBALS['browser']->isBrowser('msie') && $GLOBALS['browser']->getMajor() < 7) {
+            $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/ie6_or_less.css" rel="stylesheet" type="text/css" />' . "\n";
             if ($GLOBALS['browser']->getPlatform() == 'mac') {
                 $html .= '<link href="' . $GLOBALS['registry']->get('themesuri', 'horde') . '/ie5mac.css" rel="stylesheet" type="text/css" />' . "\n";
             }
@@ -1092,15 +1208,25 @@ HTML;
 
     /**
      * Sets a custom session handler up, if there is one.
+     * If the global variable 'session_cache_limiter' is defined, its value 
+     * will override the cache limiter setting found in the configuration
+     * file.
      */
     function setupSessionHandler()
     {
         global $conf;
 
         ini_set('url_rewriter.tags', 0);
+        if (!empty($conf['session']['use_only_cookies'])) {
+            ini_set('session.use_only_cookies', 1);
+            if (!empty($conf['cookie']['domain']) &&
+                strpos($conf['server']['name'], '.') === false) {
+                Horde::fatal('Session cookies will not work without a FQDN and with a non-empty cookie domain. Either use a fully qualified domain name like "http://www.example.com" instead of "http://example" only, or set the cookie domain in the Horde configuration to an empty value, or enable non-cookie (url-based) sessions in the Horde configuration.', __FILE__, __LINE__);
+            }
+        }
         session_set_cookie_params($conf['session']['timeout'],
                                   $conf['cookie']['path'], $conf['cookie']['domain'], $conf['use_ssl'] == 1 ? 1 : 0);
-        session_cache_limiter($conf['session']['cache_limiter']);
+        session_cache_limiter(Util::nonInputVar('session_cache_limiter', $conf['session']['cache_limiter']));
         session_name(urlencode($conf['session']['name']));
 
         $type = !empty($conf['sessionhandler']['type']) ? $conf['sessionhandler']['type'] : 'none';
@@ -1230,8 +1356,11 @@ HTML;
         }
 
         if (isset($GLOBALS['nls']['multibyte'][NLS::getCharset(true)])) {
-            return $stripped_label . '(' . '<span class="accessKey">' .
-                strtoupper($accessKey) . '</span>' . ')';
+            /* Prefix parenthesis with the UTF-8 representation of the LRO
+             * (Left-to-Right-Override) Unicode codepoint U+202D. */
+            $prefix = NLS::getCharset() == 'UTF-8' ? "\xe2\x80\xad" : '';
+            return $stripped_label . $prefix . '(<span class="accessKey">'
+                . strtoupper($accessKey) . '</span>' . ')';
         } else {
             return str_replace('_' . $accessKey, '<span class="accessKey">' . $accessKey . '</span>', $label);
         }
@@ -1298,55 +1427,12 @@ HTML;
         }
 
         $url = $GLOBALS['registry']->get('webroot', 'horde') . '/login.php';
-        $url = Util::addParameter($url, 'url', Horde::selfUrl(true));
+        $url = Util::addParameter($url, array('url' => Horde::selfUrl(true), 'nosidebar' => 1), null, false);
         $url = Auth::addLogoutParameters($url);
         header('Location: ' . Horde::url($url, true));
         exit;
     }
 
-    /**
-     * Uses DOM Tooltips (via javascript) to display the 'title'
-     * attribute for Horde::link() calls.
-     *
-     * If using this function, the following function must be called:
-     *   Horde::addScriptFile('tooltip.js', 'horde', true);
-     *
-     * @param string $url        The full URL to be linked to
-     * @param string $status     The JavaScript mouse-over string
-     * @param string $class      The CSS class of the link
-     * @param string $target     The window target to point to.
-     * @param string $onclick    JavaScript action for the 'onclick' event.
-     * @param string $title      The link title (tooltip).
-     * @param string $accesskey  The access key to use.
-     * @param array $attributes  Any other name/value pairs to add to the <a>
-     *                           tag.
-     *
-     * @return string  The full <a href> tag.
-     */
-    function linkTooltip($url, $status = '', $class = '', $target = '',
-                         $onclick = '', $title = '', $accesskey = '',
-                         $attributes = array())
-    {
-        $url = substr(Horde::link($url, null, $class, $target, $onclick, null, $accesskey, $attributes), 0, -1);
-
-        if (strlen($status)) {
-            $status = @htmlspecialchars(addslashes($status), ENT_QUOTES, NLS::getCharset());
-        }
-
-        $title = trim($title);
-        if (strlen($title)) {
-            require_once 'Horde/Text/Filter.php';
-            $title = Text_Filter::filter($title, 'text2html', array('parselevel' => TEXT_HTML_NOHTML, 'charset' => '', 'class' => ''));
-            $title = '<pre>' . strtr(addslashes($title), array("\r" => '', "\n" => '')) . '</pre>';
-            $url .= ' onmouseover="tooltipLink(\'' . @htmlspecialchars($title, ENT_QUOTES, NLS::getCharset()) . '\', \'' . $status . '\'); return true;" onmouseout="tooltipClose();"';
-        } else {
-            $url .= ' onmouseout="window.status=\'\';" onmouseover="window.status=\'' . $status . '\'; return true;"';
-        }
-        $url .= '>';
-
-        return $url;
-    }
-
     /**
      * Provides a standardised function to call a Horde hook, checking whether
      * a hook config file exists and whether the function itself exists. If
@@ -1391,59 +1477,32 @@ HTML;
     }
 
     /**
-     * Return the initial page to load when accessing Horde.
+     * Returns the specified permission for the current user.
      *
-     * @return string  The URL of the initial page.
-     */
-    function initialPage()
-    {
-        global $prefs, $registry, $perms;
-
-        $initial_app = $prefs->getValue('initial_application');
-        if (!empty($initial_app) && $registry->hasPermission($initial_app)) {
-            return Horde::url($registry->getInitialPage($initial_app), true);
-        } elseif (isset($registry->applications['horde']['initial_page'])) {
-            return Horde::applicationUrl($registry->applications['horde']['initial_page'], true);
-        } elseif (Auth::getAuth()) {
-            return Horde::applicationUrl('services/portal/', true);
-        } else {
-            return Horde::applicationUrl('login.php', true);
-        }
-    }
-
-    /**
-     * Returns an array of available menu items when in a Horde script.
+     * @since Horde 3.1
      *
-     * @return array  Available menu items.
+     * @param string $permission  A permission, currently only 'max_blocks'.
+     *
+     * @return mixed  The value of the specified permission.
      */
-    function getHordeMenu()
+    function hasPermission($permission)
     {
-        global $registry, $prefs;
+        global $perms;
 
-        $menu = array();
-
-        /* The home page button. If an initial application has been set in the
-         * prefs this will default to that application. Otherwise it will go
-         * to the portal screen. */
-        $initial_app = $prefs->getValue('initial_application');
-        if (!empty($initial_app)) {
-            $url = Horde::url($registry->getInitialPage($initial_app));
-        } else {
-            $url = Horde::applicationUrl('services/portal/');
-        }
-        $menu[] = array('url' => $url,
-                        'text' =>  _("Home"),
-                        'icon' => 'horde.png',
-                        'icon_path' => $registry->getImageDir());
-
-        if (Auth::isAdmin()) {
-            $menu[] = array('url' => Horde::applicationUrl('admin/'),
-                            'text' =>  _("_Administration"),
-                            'icon' => 'administration.png',
-                            'icon_path' => $registry->getImageDir());
+        if (!$perms->exists('horde:' . $permission)) {
+            return true;
         }
 
-        return $menu;
+        $allowed = $perms->getPermissions('horde:' . $permission);
+        if (is_array($allowed)) {
+            switch ($permission) {
+            case 'max_blocks':
+                $allowed = array_reduce($allowed, create_function('$a, $b', 'return max($a, $b);'), 0);
+                break;
+            }
+        }
+
+        return $allowed;
     }
 
 }
