@@ -19,9 +19,9 @@
  *                  DEFAULT: NONE (system default will be used)</pre>
  *
  *
- * $Horde: framework/Auth/Auth/ldap.php,v 1.47.10.19 2006/08/14 02:48:48 chuck Exp $
+ * $Horde: framework/Auth/Auth/ldap.php,v 1.47.10.21 2007/01/02 13:54:07 jan Exp $
  *
- * Copyright 1999-2006 Jon Parise <jon@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -299,7 +299,7 @@ class Auth_ldap extends Auth {
     function addUser($userId, $credentials)
     {
         if ($this->_params['ad']) {
-           return PEAR::raiseError(_("Auth_ldap: Adding users is not supported for Active Directory"), __FILE__, __LINE__);
+           return PEAR::raiseError(_("Auth_ldap: Adding users is not supported for Active Directory"));
         }
 
         /* Connect to the LDAP server. */
@@ -354,7 +354,7 @@ class Auth_ldap extends Auth {
     function removeUser($userId)
     {
         if ($this->_params['ad']) {
-           return PEAR::raiseError(_("Auth_ldap: Removing users is not supported for Active Directory"), __FILE__, __LINE__);
+           return PEAR::raiseError(_("Auth_ldap: Removing users is not supported for Active Directory"));
         }
 
         /* Connect to the LDAP server. */
@@ -382,7 +382,7 @@ class Auth_ldap extends Auth {
 
         $success = @ldap_delete($this->_ds,$dn);
         if (!$success) {
-           return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to remove user %s"), $userId), __FILE__, __LINE__);
+           return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to remove user %s"), $userId));
         }
 
         @ldap_close($this->_ds);
@@ -402,13 +402,13 @@ class Auth_ldap extends Auth {
     function updateUser($oldID, $newID, $credentials)
     {
         if ($this->_params['ad']) {
-           return PEAR::raiseError(_("Auth_ldap: Updating users is not supported for Active Directory."), __FILE__, __LINE__);
+           return PEAR::raiseError(_("Auth_ldap: Updating users is not supported for Active Directory."));
         }
 
         /* Connect to the LDAP server. */
         $success = $this->_connect();
         if (is_a($success, 'PEAR_Error')) {
-            return PEAR::raiseError($success->getMessage(), __FILE__, __LINE__);
+            return PEAR::raiseError($success->getMessage());
         }
 
         global $conf;
@@ -426,7 +426,7 @@ class Auth_ldap extends Auth {
             $dn = $this->_findDN($oldID);
 
             if (is_a($dn, 'PEAR_Error')) {
-                return PEAR::raiseError($success->getMessage(), __FILE__, __LINE__);
+                return PEAR::raiseError($success->getMessage());
             } else {
                 $olddn = $dn;
                 $newdn = preg_replace('/uid=.*?,/','uid='.$newID.',',$dn,1);
@@ -496,7 +496,7 @@ class Auth_ldap extends Auth {
         }
 
         if (!$success) {
-            return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to update user %s"), $newID), __FILE__, __LINE__);
+            return PEAR::raiseError(sprintf(_("Auth_ldap: Unable to update user %s"), $newID));
         }
 
         @ldap_close($this->_ds);
@@ -514,7 +514,7 @@ class Auth_ldap extends Auth {
         /* Connect to the LDAP server. */
         $success = $this->_connect();
         if (is_a($success, 'PEAR_Error')) {
-            return PEAR::raiseError($success->getMessage(), __FILE__, __LINE__);
+            return PEAR::raiseError($success->getMessage());
         }
 
         if (!empty($this->_params['filter'])) {
