@@ -6,7 +6,7 @@ require_once 'Horde/Array.php';
  * The Horde_Block_Layout_Manager class allows manipulation of
  * Horde_Block layouts.
  *
- * $Horde: framework/Block/Block/Layout/Manager.php,v 1.4.2.5 2007/02/07 19:46:12 jan Exp $
+ * $Horde: framework/Block/Block/Layout/Manager.php,v 1.4.2.6 2007/11/22 00:16:14 jan Exp $
  *
  * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
  * Copyright 2003-2007 Jan Schneider <jan@horde.org>
@@ -80,13 +80,13 @@ class Horde_Block_Layout_Manager {
     /**
      * Constructor.
      *
-     * @param Horde_Block_Collection &$collection
-     * @param string $data
+     * @param Horde_Block_Collection $collection
+     * @param array $data
      */
-    function Horde_Block_Layout_Manager(&$collection, $data = '')
+    function Horde_Block_Layout_Manager(&$collection, $layout = array())
     {
         $this->_collection = &$collection;
-        $this->unserialize($data);
+        $this->_layout = $layout;
 
         // Fill the _covered caches and empty rows.
         $rows = count($this->_layout);
