@@ -7,10 +7,10 @@
  *   'dir'     The directory to store the cache files in.
  *   'prefix'  The filename prefix to use for the cache files.</pre>
  *
- * $Horde: framework/Cache/Cache/file.php,v 1.28.10.9 2006/01/01 21:28:10 jan Exp $
+ * $Horde: framework/Cache/Cache/file.php,v 1.28.10.14 2007/03/09 04:54:37 slusarz Exp $
  *
- * Copyright 1999-2006 Anil Madhavapeddy <anil@recoil.org>
- * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Anil Madhavapeddy <anil@recoil.org>
+ * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -74,10 +74,7 @@ class Horde_Cache_file extends Horde_Cache {
             if (!$size) {
                 return '';
             }
-            $data = file_get_contents($filename);
-            if ($data !== false) {
-                return $data;
-            }
+            return file_get_contents($filename);
         }
 
         /* Nothing cached, return failure. */
@@ -88,7 +85,7 @@ class Horde_Cache_file extends Horde_Cache {
      * Attempts to store data to the filesystem.
      *
      * @param string $key   Cache key.
-     * @param mixed  $data  Data to store in the cache.
+     * @param mixed  $data  Data to store in the cache. (MUST BE A STRING)
      *
      * @return boolean  True on success, false on failure.
      */
@@ -98,15 +95,22 @@ class Horde_Cache_file extends Horde_Cache {
         $filename = $this->_keyToFile($key);
         $tmp_file = Util::getTempFile('HordeCache', true, $this->_dir);
 
-        if ($fd = fopen($tmp_file, 'w')) {
-            if (fwrite($fd, $data) < strlen($data)) {
-                fclose($fd);
+        if (function_exists('file_put_contents')) {
+            if (file_put_contents($tmp_file, $data) === false) {
                 return false;
-            } else {
-                fclose($fd);
-                @rename($tmp_file, $filename);
             }
+        } elseif ($fd = fopen($tmp_file, 'w')) {
+            $res = fwrite($fd, $data);
+            fclose($fd);
+            if ($res < strlen($data)) {
+                return false;
+            }
+        } else {
+            return false;
         }
+
+        @rename($tmp_file, $filename);
+        return true;
     }
 
     /**
@@ -115,21 +119,17 @@ class Horde_Cache_file extends Horde_Cache {
      * @param string  $key       Cache key to output.
      * @param integer $lifetime  Lifetime of the data in seconds.
      *
-     * @return mixed  Cached data, or false if none was found.
+     * @return boolean  True if output or false if no object was found.
      */
     function output($key, $lifetime = 1)
     {
-        if ($this->exists($key, $lifetime)) {
-            $filename = $this->_keyToFile($key);
-            $fd = @fopen($filename, 'r');
-            if ($fd) {
-                fpassthru($fd);
-                return true;
-            }
+        $data = $this->get($key, $lifetime);
+        if ($data === false) {
+            return false;
+        } else {
+            echo $data;
+            return true;
         }
-
-        /* Nothing cached, return failure. */
-        return false;
     }
 
     /**
@@ -194,7 +194,7 @@ class Horde_Cache_file extends Horde_Cache {
     /**
      * Do any garbage collection needed for the driver.
      *
-     * @private
+     * @access private
      *
      * @param integer $secs  The minimum amount of time (in seconds) required
      *                       before a cache item is removed.
@@ -222,9 +222,13 @@ class Horde_Cache_file extends Horde_Cache {
         }
         $d->close();
  
-        $fp = fopen($filename, 'w');
-        fwrite($fp, time());
-        fclose($fp);
+        if (function_exists('file_put_contents')) {
+            file_put_contents($filename, time());
+        } else {
+            $fp = fopen($filename, 'w');
+            fwrite($fp, time());
+            fclose($fp);
+        }
     }
 
 }
