@@ -1,4 +1,7 @@
 <?php
+
+require_once 'Horde/Form.php';
+
 /**
  * The Config:: package provides a framework for managing the
  * configuration of Horde applications, writing conf.php files from
@@ -13,9 +16,9 @@
  *
  *   http://alexandre.alapetite.net/doc-alex/domxml-php4-php5/
  *
- * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.10 2005/06/23 20:18:07 selsky Exp $
+ * $Horde: framework/Horde/Horde/Config.php,v 1.80.2.30 2007/01/12 18:28:37 chuck Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -109,23 +112,30 @@ class Horde_Config {
     }
 
     /**
-     * Reads the application's conf.xml file and builds an associative
-     * array from its XML tree.
+     * Reads the application's conf.xml file and builds an associative array
+     * from its XML tree.
+     *
+     * @param array $custom_conf   Any settings that shall be included in the
+     *                             generated configuration.
      *
      * @return array  An associative array representing the configuration tree.
      */
-    function readXMLConfig()
+    function readXMLConfig($custom_conf = null)
     {
-        if (is_null($this->_xmlConfigTree)) {
+        if (is_null($this->_xmlConfigTree) || $custom_conf) {
             require_once 'Horde/Text.php';
 
             global $registry;
             $path = $registry->get('fileroot', $this->_app) . '/config';
 
-            /* Fetch the current conf.php contents. */
-            @eval($this->getPHPConfig());
-            if (isset($conf)) {
-                $this->_currentConfig = $conf;
+            if ($custom_conf) {
+                $this->_currentConfig = $custom_conf;
+            } else {
+                /* Fetch the current conf.php contents. */
+                @eval($this->getPHPConfig());
+                if (isset($conf)) {
+                    $this->_currentConfig = $conf;
+                }
             }
 
             /* Load the DOM object. */
@@ -164,19 +174,20 @@ class Horde_Config {
         if (is_null($this->_oldConfig)) {
             global $registry;
             $path = $registry->get('fileroot', $this->_app) . '/config';
-            $size = @filesize($path . '/conf.php');
-            if ($size && is_resource($fp = @fopen($path . '/conf.php', 'r'))) {
-                $this->_oldConfig = @fread($fp, $size);
-                $this->_oldConfig = preg_replace('/<\?php\n?/', '', $this->_oldConfig);
-                $pos = strpos($this->_oldConfig, $this->_configBegin);
-                if ($pos !== false) {
-                    $this->_preConfig = substr($this->_oldConfig, 0, $pos);
-                    $this->_oldConfig = substr($this->_oldConfig, $pos);
-                }
-                $pos = strpos($this->_oldConfig, $this->_configEnd);
-                if ($pos !== false) {
-                    $this->_postConfig = substr($this->_oldConfig, $pos + strlen($this->_configEnd));
-                    $this->_oldConfig = substr($this->_oldConfig, 0, $pos);
+            if (file_exists($path . '/conf.php')) {
+                $this->_oldConfig = file_get_contents($path . '/conf.php');
+                if (!empty($this->_oldConfig)) {
+                    $this->_oldConfig = preg_replace('/<\?php\n?/', '', $this->_oldConfig);
+                    $pos = strpos($this->_oldConfig, $this->_configBegin);
+                    if ($pos !== false) {
+                        $this->_preConfig = substr($this->_oldConfig, 0, $pos);
+                        $this->_oldConfig = substr($this->_oldConfig, $pos);
+                    }
+                    $pos = strpos($this->_oldConfig, $this->_configEnd);
+                    if ($pos !== false) {
+                        $this->_postConfig = substr($this->_oldConfig, $pos + strlen($this->_configEnd));
+                        $this->_oldConfig = substr($this->_oldConfig, 0, $pos);
+                    }
                 }
             } else {
                 $this->_oldConfig = '';
@@ -189,12 +200,14 @@ class Horde_Config {
      * Generates the content of the application's configuration file.
      *
      * @param Variables $formvars  The processed configuration form data.
+     * @param array $custom_conf   Any settings that shall be included in the
+     *                             generated configuration.
      *
      * @return string  The content of the generated configuration file.
      */
-    function generatePHPConfig($formvars)
+    function generatePHPConfig($formvars, $custom_conf = null)
     {
-        $this->readXMLConfig();
+        $this->readXMLConfig($custom_conf);
         $this->getPHPConfig();
 
         $this->_phpConfig = "<?php\n";
@@ -299,7 +312,7 @@ class Horde_Config {
                     break;
 
                 case 'int':
-                    if ($val != '') {
+                    if ($val !== '') {
                         $value = (int)$val;
                     }
                     break;
@@ -382,13 +395,20 @@ class Horde_Config {
                 break;
 
             case 'configswitch':
+                $values = $this->_getSwitchValues($node, $ctx);
                 if ($quote) {
                     $default = $this->_default($curctx, $this->_getNodeOnlyText($node));
                 } else {
                     $default = $this->_defaultRaw($curctx, $this->_getNodeOnlyText($node));
                 }
+                if ($default === '') {
+                    $default = key($values);
+                }
+                if (is_bool($default)) {
+                    $default = $default ? 'true' : 'false';
+                }
                 $conf[$name] = array('desc' => $desc,
-                                     'switch' => $this->_getSwitchValues($node, $ctx),
+                                     'switch' => $values,
                                      'default' => $default);
                 break;
 
@@ -399,6 +419,12 @@ class Horde_Config {
                 } else {
                     $default = $this->_defaultRaw($curctx, $this->_getNodeOnlyText($node));
                 }
+                if ($default === '') {
+                    $default = key($values);
+                }
+                if (is_bool($default)) {
+                    $default = $default ? 'true' : 'false';
+                }
                 $conf[$name] = array('_type' => 'enum',
                                      'required' => $required,
                                      'quote' => $quote,
@@ -588,6 +614,7 @@ class Horde_Config {
                 'tcp' => array(
                     'desc' => 'TCP/IP',
                     'fields' => array(
+                        'hostspec' => $hostspec,
                         'port' => $port))));
         $mysql_protocol = $protocol;
         $mysql_protocol['switch']['tcp']['fields']['port']['default'] = $this->_default($ctx . '|port', 3306);
@@ -619,8 +646,11 @@ class Horde_Config {
         $custom_fields = array(
             'required' => true,
             'desc' => 'What database backend should we use?',
-            'default' => $this->_default($ctx . '|phptype', 'mysql'),
+            'default' => $this->_default($ctx . '|phptype', 'false'),
             'switch' => array(
+                'false' => array(
+                    'desc' => '[None]',
+                    'fields' => array()),
                 'dbase' => array(
                     'desc' => 'dBase',
                     'fields' => array(
@@ -636,7 +666,7 @@ class Horde_Config {
                                 0 => 'Read Only',
                                 2 => 'Read Write'),
                             'default' => $this->_default($ctx . '|mode', 2)),
-                        
+
                         'charset' => $charset)),
                 'ibase' => array(
                     'desc' => 'Firebird/InterBase',
@@ -711,21 +741,17 @@ class Horde_Config {
                     'desc' => 'MySQL',
                     'fields' => array(
                         'persistent' => $persistent,
-                        'hostspec' => $hostspec,
                         'username' => $username,
                         'password' => $password,
                         'protocol' => $mysql_protocol,
                         'database' => $database,
                         'charset' => $charset)),
                 'mysqli' => array(
-                    'desc' => 'MySQL improved (4.1+)',
+                    'desc' => 'MySQL (mysqli)',
                     'fields' => array(
-                        'persistent' => $persistent,
-                        'hostspec' => $hostspec,
                         'username' => $username,
                         'password' => $password,
-                        'port' => $port,
-                        'socket' => $socket,
+                        'protocol' => $mysql_protocol,
                         'database' => $database,
                         'charset' => $charset)),
                 'oci8' => array(
@@ -754,7 +780,7 @@ class Horde_Config {
                                 'mssql' => 'MS SQL Server',
                                 'sybase' => 'Sybase',
                                 'mysql' => 'MySQL',
-                                'mysqli' => 'MySQL improved',
+                                'mysqli' => 'MySQL (mysqli)',
                                 ),
                             'default' => $this->_default($ctx . '|dbsyntax', 'sql92')),
                         'cursor' => array(
@@ -789,7 +815,7 @@ class Horde_Config {
                             'desc' => 'Absolute path to the database file',
                             'default' => $this->_default($ctx . '|database', '')),
                         'mode' => array(
-                            '_type' => 'octal',
+                            '_type' => 'text',
                             'desc' => 'The mode to open the file with',
                             'default' => $this->_default($ctx . '|mode', '0644')),
                         'charset' => $charset)),
@@ -859,6 +885,9 @@ class Horde_Config {
                 'desc' => 'What VFS driver should we use?',
                 'default' => $this->_default($ctx . '|' . $node->get_attribute('switchname'), 'horde'),
                 'switch' => array(
+                    'none' => array(
+                        'desc' => 'None',
+                        'fields' => array()),
                     'horde' => array(
                         'desc' => 'Horde defaults',
                         'fields' => array()),
@@ -937,7 +966,7 @@ class Horde_Config {
     function _defaultRaw($ctx, $default)
     {
         $ctx = explode('|', $ctx);
-        $pattern = '/^\$conf\[\'' . implode("'\]\['", $ctx) . '\'\] = (.*);$/m';
+        $pattern = '/^\$conf\[\'' . implode("'\]\['", $ctx) . '\'\] = (.*);\r?$/m';
         if (preg_match($pattern, $this->getPHPConfig(), $matches)) {
             return $matches[1];
         }
@@ -1067,10 +1096,14 @@ class Horde_Config {
             break;
 
         case 'list-horde-languages':
-            global $nls;
-            return $nls['languages'];
+            return array_map(create_function('$val', 'return preg_replace(array("/&#x([0-9a-f]{4});/ie", "/(&[^;]+;)/e"), array("String::convertCharset(pack(\"H*\", \"$1\"), \"ucs-2\", \"' . NLS::getCharset() . '\")", "String::convertCharset(html_entity_decode(\"$1\", ENT_COMPAT, \"iso-8859-1\"), \"iso-8859-1\", \"' . NLS::getCharset() . '\")"), $val);'), $GLOBALS['nls']['languages']);
             break;
 
+        case 'list-blocks':
+            require_once 'Horde/Block/Collection.php';
+            $collection = &Horde_Block_Collection::singleton('portal');
+            return $collection->getBlocksList();
+
         case 'list-client-fields':
             global $registry;
             $f = array();
@@ -1208,6 +1241,9 @@ class ConfigForm extends Horde_Form {
             } elseif (isset($configitem['_type'])) {
                 $required = (isset($configitem['required'])) ? $configitem['required'] : true;
                 $type = $configitem['_type'];
+                // FIXME: multienum fields can well be required, meaning that
+                // you need to select at least one entry. Changing this before
+                // Horde 4.0 would break a lot of configuration files though.
                 if ($type == 'multienum' || $type == 'header' ||
                     $type == 'description') {
                     $required = false;
