@@ -6,7 +6,7 @@ require_once 'Horde/Crypt.php';
  * Horde_Crypt_smime:: provides a framework for Horde applications to
  * interact with the OpenSSL library and implement S/MIME.
  *
- * $Horde: framework/Crypt/Crypt/smime.php,v 1.49.2.13 2007/01/02 13:54:12 jan Exp $
+ * $Horde: framework/Crypt/Crypt/smime.php,v 1.49.2.14 2007/08/15 08:51:52 selsky Exp $
  *
  * Copyright 2002-2007 Mike Cochrane <mike@graftonhall.co.nz>
  *
@@ -361,7 +361,7 @@ class Horde_Crypt_smime extends Horde_Crypt {
 
         /* Check for required parameters. */
         if (!isset($params['pubkey'])) {
-            return PEAR::raiseError(_("A public SMIME key is required to encrypt a message."), 'horde.error');
+            return PEAR::raiseError(_("A public S/MIME key is required to encrypt a message."), 'horde.error');
         }
 
         /* Create temp files for input/output. */
