@@ -5,9 +5,9 @@
  * The Editor:: package provides an rte editor for the Horde
  * Application Framework.
  *
- * $Horde: framework/Editor/Editor.php,v 1.8.10.5 2005/07/03 05:48:02 selsky Exp $
+ * $Horde: framework/Editor/Editor.php,v 1.8.10.9 2007/01/02 13:54:15 jan Exp $
  *
- * Copyright 2003-2005 Nuno Loureiro <nuno@co.sapo.pt>
+ * Copyright 2003-2007 Nuno Loureiro <nuno@co.sapo.pt>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -18,6 +18,13 @@
  */
 class Horde_Editor {
 
+    /**
+     * Constructor.
+     */
+    function Horde_Editor($params = array())
+    {
+    }
+
     /**
      * Attempts to return a concrete Horde_Editor instance based on $driver.
      *
@@ -40,7 +47,8 @@ class Horde_Editor {
 
         $driver = basename($driver);
         if (empty($driver) || $driver == 'none') {
-            return $editor = &new Horde_Editor($params);
+            $editor = &new Horde_Editor($params);
+            return $editor;
         }
 
         if (is_null($params) && class_exists('Horde')) {
@@ -58,10 +66,12 @@ class Horde_Editor {
 
         $class = 'Horde_Editor_' . $driver;
         if (class_exists($class)) {
-            return $editor = &new $class($params);
+            $editor = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $editor = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $editor;
     }
 
     /**
