@@ -2,15 +2,14 @@
 /**
  * Bar graph implementation for the Horde_Graph package.
  *
- * $Horde: framework/Graph/Graph/Plot/bar.php,v 1.5.12.1 2005/01/03 12:19:00 jan Exp $
+ * $Horde: framework/Graph/Graph/Plot/bar.php,v 1.5.12.5 2007/01/02 13:54:20 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
- * @version $Revision: 1.5.12.1 $
  * @since   Horde 3.0
  * @package Horde_Graph
  */
@@ -34,7 +33,7 @@ class Horde_Graph_Plot_bar {
         }
     }
 
-    function draw()
+    function draw($minY = false)
     {
         $data = $this->graph->_data['y'][$this->_dataset];
 
@@ -45,12 +44,14 @@ class Horde_Graph_Plot_bar {
         $count = count($data);
         for ($i = 0; $i < $count; $i++) {
             $x = $i;
-            $y = $data[$i];
-            $this->graph->translate($x, $y);
-            $x = $x - ($barWidth / 2) + $barOffset;
-            $height = $this->graph->_graphBottom - $y;
+            if ($minY === false || $data[$i] >= $minY) {
+                $y = $data[$i];
+                $this->graph->translate($x, $y);
+                $x = $x - ($barWidth / 2) + $barOffset;
+                $height = $this->graph->_graphBottom - $y;
 
-            $this->graph->img->rectangle($x, $y, $barWidth, $height, $this->_outline, $this->_color);
+                $this->graph->img->rectangle($x, $y, $barWidth, $height, $this->_outline, $this->_color);
+            }
         }
     }
 
