@@ -6,10 +6,10 @@ require_once 'Horde/History.php';
 /**
  * The Group:: class provides the Horde groups system.
  *
- * $Horde: framework/Group/Group.php,v 1.67.2.6.2.1 2005/12/17 21:25:21 chuck Exp $
+ * $Horde: framework/Group/Group.php,v 1.67.2.16 2007/01/02 13:54:20 jan Exp $
  *
- * Copyright 1999-2005 Stephane Huther <shuther@bigfoot.com>
- * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2007 Stephane Huther <shuther@bigfoot.com>
+ * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -48,15 +48,21 @@ class Group {
     /**
      * Return a new group object.
      *
-     * @param string $name  The group's name.
+     * @param string $name    The group's name.
+     * @param string $parent  The group's parent's name.
      *
      * @return DataTreeObject_Group  A new group object.
      */
-    function &newGroup($name)
+    function &newGroup($name, $parent = DATATREE_ROOT)
     {
         if (empty($name)) {
             return PEAR::raiseError(_("Group names must be non-empty"));
         }
+
+        if ($parent != DATATREE_ROOT) {
+            $name = $this->getGroupName($parent) . ':' . DataTree::encodeName($name);
+        }
+
         $group = &new DataTreeObject_Group($name);
         $group->setGroupOb($this);
         return $group;
@@ -214,19 +220,31 @@ class Group {
     /**
      * Retrieve the name of a group.
      *
-     * @param integer $groupId  The id of the group to retrieve the name for.
+     * @param integer $gid  The id of the group to retrieve the name for.
      *
      * @return string  The group's name.
      */
-    function getGroupName($groupId)
+    function getGroupName($gid)
     {
-        if (is_a($groupId, 'DataTreeObject_Group')) {
-            return $this->_datatree->getName($groupId->getId());
+        if (is_a($gid, 'DataTreeObject_Group')) {
+            return $this->_datatree->getName($gid->getId());
         } else {
-            return $this->_datatree->getName($groupId);
+            return $this->_datatree->getName($gid);
         }
     }
 
+    /**
+     * Strips all parent references off of the given group name.
+     *
+     * @param string $group  Name of the group.
+     *
+     * @return The name of the group without parents.
+     */
+    function getGroupShortName($group)
+    {
+        return $this->_datatree->getShortName($group);
+    }
+
     /**
      * Retrieve the ID of a group.
      *
@@ -258,13 +276,26 @@ class Group {
     /**
      * Get a list of the parents of a child group.
      *
-     * @param string $group The name of the child group.
+     * @param integer $gid The id of the child group.
      *
      * @return array
      */
-    function getGroupParents($group)
+    function getGroupParents($gid)
     {
-        return $this->_datatree->getParents($group);
+        $name = $this->getGroupName($gid);
+        return $this->_datatree->getParents($name);
+    }
+
+    /**
+     * Return the single parent of the given group.
+     *
+     * @param integer $gid  The DataTree ID of the child group.
+     *
+     * @return the parent of the given group.
+     */
+    function getGroupParent($gid)
+    {
+        return $this->_datatree->getParentById($gid);
     }
 
     /**
@@ -283,13 +314,16 @@ class Group {
     /**
      * Get a list of every group, in the format cid => groupname.
      *
+     * @param boolean $refresh  If true, the cached value is ignored and the group
+     *                          list is refreshed from the group backend.
+     *
      * @return array  CID => groupname hash.
      */
-    function listGroups()
+    function listGroups($refresh = false)
     {
         static $groups;
 
-        if (is_null($groups)) {
+        if ($refresh || is_null($groups)) {
             $groups = $this->_datatree->get(DATATREE_FORMAT_FLAT, DATATREE_ROOT, true);
             unset($groups[DATATREE_ROOT]);
         }
@@ -300,13 +334,13 @@ class Group {
     /**
      * Get a list of every user that is a part of this group ONLY.
      *
-     * @param string $group  The name of the group.
+     * @param integer $gid  The ID of the group.
      *
      * @return array  The user list.
      */
-    function listUsers($group)
+    function listUsers($gid)
     {
-        $groupOb = &$this->getGroup($group);
+        $groupOb = &$this->getGroupById($gid);
         if (is_a($groupOb, 'PEAR_Error')) {
             return $groupOb;
         }
@@ -323,22 +357,22 @@ class Group {
      * Get a list of every user that is part of the specified group
      * and any of its subgroups.
      *
-     * @param string $group  The name of the parent group.
+     * @param integer $group  The ID of the parent group.
      *
      * @return array  The complete user list.
      */
-    function listAllUsers($group)
+    function listAllUsers($gid)
     {
         // Get a list of every group that is a sub-group of $group.
-        $groups = $this->_datatree->get(DATATREE_FORMAT_FLAT, $group, true);
+        $groups = $this->_datatree->get(DATATREE_FORMAT_FLAT, $this->getGroupName($gid), true);
         if (is_a($groups, 'PEAR_Error')) {
             return $groups;
         }
 
-        $groups = array($group) + $groups;
+        $groups = array_keys($groups);
         $users = array();
-        foreach ($groups as $group) {
-            $users = array_merge($users, $this->listUsers($group));
+        foreach ($groups as $groupId) {
+            $users = array_merge($users, $this->listUsers($groupId));
         }
         return array_values(array_flip(array_flip($users)));
     }
@@ -363,7 +397,8 @@ class Group {
             $groups = $this->_datatree->getByAttributes($criteria);
 
             if (is_a($groups, 'PEAR_Error')) {
-                return $groups;
+                Horde::logMessage($groups, __FILE__, __LINE__, PEAR_LOG_ERROR);
+                return false;
             }
 
             if ($parentGroups) {
@@ -385,33 +420,49 @@ class Group {
     /**
      * Say if a user is a member of a group or not.
      *
-     * @param string  $user       The name of the user.
-     * @param string  $group      The name of the group.
+     * @param string $user        The name of the user.
+     * @param integer $gid        The ID of the group.
      * @param boolean $subgroups  Return true if the user is in any subgroups
-     *                            of $group, also.
+     *                            of group with ID $gid, also.
      *
      * @return boolean
      */
-    function userIsInGroup($user, $group, $subgroups = true)
+    function userIsInGroup($user, $gid, $subgroups = true)
     {
-        if (!$this->exists($group)) {
+        if (!$this->exists($this->getGroupName($gid))) {
             return false;
         } elseif ($subgroups) {
             $groups = $this->getGroupMemberships($user, true);
             if (is_a($groups, 'PEAR_Error')) {
-                return $groups;
+                Horde::logMessage($groups, __FILE__, __LINE__, PEAR_LOG_ERR);
+                return false;
             }
 
-            return !empty($groups[$this->getGroupId($group)]);
+            return !empty($groups[$gid]);
         } else {
-            $users = $this->listUsers($group);
+            $users = $this->listUsers($gid);
             if (is_a($users, 'PEAR_Error')) {
-                return $users;
+                Horde::logMessage($users, __FILE__, __LINE__, PEAR_LOG_ERR);
+                return false;
             }
             return in_array($user, $users);
         }
     }
 
+    /**
+     * Returns the DataTree level of the given group. 0 is returned for
+     * and object directly below DATATREE_ROOT.
+     *
+     * @param integer $gid  The DataTree ID of the group.
+     *
+     * @return The DataTree level of the group.
+     */
+    function getLevel($gid)
+    {
+        $name = $this->_datatree->getName($gid);
+        return substr_count($name, ':');
+    }
+
     /**
      * Attempts to return a concrete Group instance based on $driver.
      *
@@ -433,10 +484,12 @@ class Group {
         }
         $class = 'Group_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $group = &new $class($params);
         } else {
-            return PEAR::raiseError('Class definition of ' . $class . ' not found.');
+            $group = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
+
+        return $group;
     }
 
     /**
@@ -536,7 +589,8 @@ class DataTreeObject_Group extends DataTreeObject {
      */
     function save()
     {
-        $this->_groupOb->updateGroup($this);
+        return $this->_groupOb->updateGroup($this);
+
     }
 
     /**
@@ -549,8 +603,8 @@ class DataTreeObject_Group extends DataTreeObject {
     {
         $this->data['users'][$username] = 1;
         $this->_auditLog[$username] = 'addUser';
-        if ($update && $this->_groupOb->_datatree->exists($this->getName())) {
-            $this->save();
+        if ($update && $this->_groupOb->exists($this->getName())) {
+            return $this->save();
         }
     }
 
@@ -565,7 +619,7 @@ class DataTreeObject_Group extends DataTreeObject {
         unset($this->data['users'][$username]);
         $this->_auditLog[$username] = 'deleteUser';
         if ($update) {
-            $this->save();
+            return $this->save();
         }
     }
 
@@ -577,7 +631,7 @@ class DataTreeObject_Group extends DataTreeObject {
      */
     function listUsers()
     {
-        return $this->_groupOb->listUsers($this->name);
+        return $this->_groupOb->listUsers($this->getId());
     }
 
     /**
@@ -588,7 +642,7 @@ class DataTreeObject_Group extends DataTreeObject {
      */
     function listAllUsers()
     {
-        return $this->_groupOb->listAllUsers($this->name);
+        return $this->_groupOb->listAllUsers($this->getId());
     }
 
     /**
