@@ -7,9 +7,9 @@ require_once dirname(__FILE__) . '/../Image.php';
  * extension. It mainly provides some utility functions, such as the
  * ability to make pixels, for now.
  *
- * $Horde: framework/Image/Image/gd.php,v 1.48.10.12 2005/10/13 13:04:29 jan Exp $
+ * $Horde: framework/Image/Image/gd.php,v 1.48.10.17 2007/01/03 00:07:32 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -1023,10 +1023,16 @@ class Horde_Image_gd extends Horde_Image {
     {
         unset($php_errormsg);
         $track = ini_set('track_errors', 1);
-        $result = @call_user_func_array($function, $params);
+        $error_mask = E_ALL & ~E_WARNING & ~E_NOTICE;
+        if (defined('E_STRICT')) {
+            $error_mask &= ~E_STRICT;
+        }
+        error_reporting($error_mask);
+        $result = call_user_func_array($function, $params);
         if ($track !== false) {
             ini_set('track_errors', $track);
         }
+        error_reporting($GLOBALS['conf']['debug_level']);
         if (!empty($php_errormsg)) {
             require_once 'PEAR.php';
             $result = PEAR::raiseError($function . ': ' . $php_errormsg);
