@@ -5,10 +5,10 @@ require_once 'Horde/Image.php';
 /**
  * This class implements the Horde_Image:: API for ImageMagick.
  *
- * $Horde: framework/Image/Image/im.php,v 1.34.10.9 2005/06/23 20:07:19 selsky Exp $
+ * $Horde: framework/Image/Image/im.php,v 1.34.10.13 2007/01/02 13:54:23 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -222,7 +222,7 @@ class Horde_Image_im extends Horde_Image {
      */
     function grayscale()
     {
-        $this->_operations[] = '-colorspace GRAY -colors 256';
+        $this->_operations[] = '-colorspace GRAY';
     }
 
     /**
