@@ -2,15 +2,19 @@
 /**
  * The description of Horde to use in the 'User-Agent:' header.
  */
-define('HORDE_AGENT_HEADER', 'Horde Application Framework 3.0');
+define('HORDE_AGENT_HEADER', 'Horde Application Framework 3.1');
 
 /**
  * The MIME_Headers:: class contains generic functions related to
  * handling the headers of mail messages.
  *
- * $Horde: framework/MIME/MIME/Headers.php,v 1.29.10.13 2005/10/07 03:57:51 chuck Exp $
+ * The default character set to use for headers should be defined in the
+ * variable $GLOBALS['mime_headers']['default_charset'] (defaults to US-ASCII
+ * per RFC 2045).
  *
- * Copyright 2002-2005 Michael Slusarz <slusarz@bigworm.colorado.edu>
+ * $Horde: framework/MIME/MIME/Headers.php,v 1.29.10.22 2007/01/02 13:54:25 jan Exp $
+ *
+ * Copyright 2002-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -112,6 +116,9 @@ class MIME_Headers {
     {
         if (!is_null($this->_index) && empty($this->_headerText)) {
             $this->_headerText = @imap_fetchheader($this->_getStream(), $this->_index, FT_UID);
+            if (!empty($GLOBALS['mime_headers']['default_charset'])) {
+                $this->_headerText = String::convertCharset($this->_headerText, $GLOBALS['mime_headers']['default_charset']);
+            }
         }
 
         return $this->_headerText;
@@ -143,7 +150,7 @@ class MIME_Headers {
     function getHeaderObject()
     {
         if (!is_null($this->_index) && empty($this->_headerObject)) {
-            $stream = &$this->_getStream();
+            $stream = $this->_getStream();
             $this->_headerObject = @imap_headerinfo($stream, @imap_msgno($stream, $this->_index));
         }
 
@@ -223,14 +230,12 @@ class MIME_Headers {
         $return_array = array();
 
         foreach ($this->_headers as $ob) {
-            $eol = $this->getEOL();
             $header = $ob['header'];
             if (is_array($ob['value'])) {
-                require_once dirname(__FILE__) . '/../MIME.php';
-                $return_array[$header] = MIME::wrapHeaders($header, reset($ob['value']));
-                next($ob['value']);
-                while (list(,$val) = each($ob['value'])) {
-                    $return_array[$header] .= $eol . $header . ': ' . MIME::wrapHeaders($header, $val, $eol);
+                if (String::lower($header) == 'received') {
+                    $return_array[$header] = $ob['value'];
+                } else {
+                    $return_array[$header] = reset($ob['value']);
                 }
             } else {
                 $return_array[$header] = $ob['value'];
@@ -670,6 +675,8 @@ class MIME_Headers {
     /**
      * Get a header from the header object.
      *
+     * @todo Replace with getOb() from IMP's IMP_Headers for Horde 4.0.
+     *
      * @param string $field    The object field to retrieve (see
      *                         imap_headerinfo() for the list of fields).
      * @param boolean $decode  Should the return value be MIME decoded?
@@ -691,6 +698,9 @@ class MIME_Headers {
             $data = $ob->$field;
             if (!empty($decode) && !is_object($data) && !is_array($data)) {
                 include_once 'Horde/MIME.php';
+                if (!empty($GLOBALS['mime_headers']['default_charset'])) {
+                    $data = String::convertCharset($data, $GLOBALS['mime_headers']['default_charset']);
+                }
                 $data = MIME::decode($data);
             }
         }
