@@ -8,9 +8,9 @@ require_once dirname(__FILE__) . '/source.php';
  *
  * Source-highlight: http://www.gnu.org/software/src-highlite/
  *
- * $Horde: framework/MIME/MIME/Viewer/srchighlite.php,v 1.14.10.3 2005/07/03 05:04:10 selsky Exp $
+ * $Horde: framework/MIME/MIME/Viewer/srchighlite.php,v 1.14.10.9 2007/01/02 13:54:26 jan Exp $
  *
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -30,7 +30,7 @@ class MIME_Viewer_srchighlite extends MIME_Viewer_source {
      */
     function render($params = array())
     {
-        global $mime_drivers, $registry;
+        global $mime_drivers;
 
         /* Check to make sure the program actually exists. */
         if (!file_exists($mime_drivers['horde']['srchighlite']['location'])) {
@@ -90,30 +90,19 @@ class MIME_Viewer_srchighlite extends MIME_Viewer_source {
             // break;
         }
 
-        // Educated Guess at whether we are inline or not
-        $inline = headers_sent();
-        $html = '';
-
-        if (!$inline) {
-            $html .= Util::bufferOutput('require', $registry->get('templates', 'horde') . '/common-header.inc');
-            $html .= '<div style="background-color:white">';
-        }
-
         /* Execute Source-Highlite. */
         exec($mime_drivers['horde']['srchighlite']['location'] . " --src-lang $lang --out-format xhtml --input $tmpin --output $tmpout");
-        $fp = @fopen($tmpout, 'r');
-        $data = @fread($fp, filesize($tmpout));
-        @fclose($fp);
+        $results = file_get_contents($tmpout);
         unlink($tmpout);
 
-        $html .= $this->lineNumber($data);
-
-        if (!$inline) {
-            $html .= '</div>';
-            $html .= Util::bufferOutput('require', $registry->get('templates', 'horde') . '/common-footer.inc');
+        /* Educated Guess at whether we are inline or not. */
+        if (headers_sent() || ob_get_length()) {
+            return $this->lineNumber($results);
+        } else {
+            return Util::bufferOutput('require', $GLOBALS['registry']->get('templates', 'horde') . '/common-header.inc') .
+                $this->lineNumber($results) .
+                Util::bufferOutput('require', $GLOBALS['registry']->get('templates', 'horde') . '/common-footer.inc');
         }
-
-        return $html;
     }
 
 }
