@@ -64,7 +64,7 @@ $GLOBALS['intervals'] = &$intervals;
  * The Maintenance:: class provides a set of methods for dealing with
  * maintenance operations run upon login to Horde applications.
  *
- * $Horde: framework/Maintenance/Maintenance.php,v 1.52.10.16 2007/02/09 12:32:10 jan Exp $
+ * $Horde: framework/Maintenance/Maintenance.php,v 1.52.10.17 2007/03/29 12:34:32 jan Exp $
  *
  * Copyright 2001-2007 Michael Slusarz <slusarz@bigworm.colorado.edu>
  *
@@ -173,7 +173,9 @@ class Maintenance {
         if (!Util::getFormData(MAINTENANCE_DONE_PARAM)) {
             /* Determine if we should redirect to the maintenance page. */
             if ($this->_needMaintenancePage() !== null) {
-                header('Location: ' . $this->_getMaintenanceURL());
+                $url = Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/maintenance.php', true);
+                $url = Util::addParameter($url, array('domaintenance' => 1, 'module' => $this->_tasklist->getModule()), null, false);
+                header('Location: ' . $url);
                 exit;
             }
         }
@@ -282,12 +284,17 @@ class Maintenance {
      */
     function getPostData()
     {
-        $text = '';
+        $data = $this->_tasklist->getPostData();
+        $data['domaintenance'] = 1;
+        if ($this->_needMaintenancePage() !== null) {
+            $data['module'] = $this->_tasklist->getModule();
+        } else {
+            $data[MAINTENANCE_DONE_PARAM] = 1;
+        }
 
-        if (($data = $this->_tasklist->getPostData())) {
-            foreach ($data as $name => $val) {
-                $text .= '<input type="hidden" name="' . htmlspecialchars($name) . '" value="' . htmlspecialchars($val) . '" />' . "\n";
-            }
+        $text = '';
+        foreach ($data as $name => $val) {
+            $text .= '<input type="hidden" name="' . htmlspecialchars($name) . '" value="' . htmlspecialchars($val) . '" />' . "\n";
         }
 
         return $text;
@@ -300,11 +307,11 @@ class Maintenance {
      */
     function getMaintenanceFormURL()
     {
-       if ($this->_needMaintenancePage() !== null) {
-           return $this->_getMaintenanceURL();
-       } else {
-           return $this->_getInitialPageURL();
-       }
+        if ($this->_needMaintenancePage() !== null) {
+            return Horde::url($GLOBALS['registry']->get('webroot', 'horde') . '/services/maintenance.php', true);
+        } else {
+            return $this->_tasklist->getTarget();
+        }
     }
 
     /**
@@ -426,36 +433,6 @@ class Maintenance {
         return $this->_taskCache[$modname];
     }
 
-    /**
-     * Get the URL for the initial page.
-     *
-     * @access private
-     *
-     * @return string  The URL for the initial page.
-     */
-    function _getInitialPageURL()
-    {
-        return Util::addParameter($this->_tasklist->getTarget(), array(MAINTENANCE_DONE_PARAM => 1, 'domaintenance' => 1), null, false);
-    }
-
-    /**
-     * Get the URL for the maintenance page.
-     *
-     * @access private
-     *
-     * @return string  The URL for the maintenance page.
-     */
-    function _getMaintenanceURL()
-    {
-        global $registry;
-
-        /* We need to pass the cache ID and 'module' parameters. */
-        $url = Horde::url($registry->get('webroot', 'horde') . '/services/maintenance.php', true);
-        $url = Util::addParameter($url, array('module' => $this->_tasklist->getModule(), 'domaintenance' => 1), null, false);
-
-        return $url;
-    }
-
     /**
      * Register the shutdown function for storing the maintenance
      * tasklist.
@@ -590,7 +567,7 @@ class Maintenance_Tasklist {
      *
      * @var array
      */
-    var $_postdata;
+    var $_postdata = array();
 
     /**
      * The list of tasks to run during this login.
