@@ -6,10 +6,10 @@ require_once 'Horde/DataTree.php';
  * The Perms_datatree:: class provides a DataTree driver for the Horde
  * permissions system.
  *
- * $Horde: framework/Perms/Perms/datatree.php,v 1.6.2.7 2005/07/18 10:08:51 jan Exp $
+ * $Horde: framework/Perms/Perms/datatree.php,v 1.6.2.13 2007/01/02 13:54:34 jan Exp $
  *
- * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2004-2005 Jan Schneider <jan@horde.org>
+ * Copyright 2001-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2004-2007 Jan Schneider <jan@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -46,6 +46,37 @@ class Perms_datatree extends Perms {
                                                             array('group' => 'horde.perms')));
     }
 
+    /**
+     * Returns the available permissions for a given level.
+     *
+     * @param string $name  The permission's name.
+     *
+     * @return array  An array of available permissions and their titles.
+     */
+    function getAvailable($name)
+    {
+        if ($name == DATATREE_ROOT) {
+            $name = '';
+        }
+        return parent::getAvailable($name);
+    }
+
+    /**
+     * Given a permission name, returns the title for that permission by
+     * looking it up in the applications's permission api.
+     *
+     * @param string $name  The permissions's name.
+     *
+     * @return string  The title for the permission.
+     */
+    function getTitle($name)
+    {
+        if ($name == DATATREE_ROOT) {
+            return _("All Permissions");
+        }
+        return parent::getTitle($name);
+    }
+
     /**
      * Returns a new permissions object.
      *
@@ -59,7 +90,20 @@ class Perms_datatree extends Perms {
             return PEAR::raiseError('Permission names must be non-empty');
         }
 
-        $perm = &new DataTreeObject_Permission($name);
+        $type = 'matrix';
+        $params = null;
+        if ($pos = strpos($name, ':')) {
+            $info = $this->getApplicationPermissions(substr($name, 0, $pos));
+            if (!is_a($info, 'PEAR_Error')) {
+                if (isset($info['type']) && isset($info['type'][$name])) {
+                    $type = $info['type'][$name];
+                }
+                if (isset($info['params']) && isset($info['params'][$name])) {
+                    $params = $info['params'][$name];
+                }
+            }
+        }
+        $perm = &new DataTreeObject_Permission($name, $type, $params);
         $perm->setDataTree($this->_datatree);
 
         return $perm;
@@ -93,7 +137,11 @@ class Perms_datatree extends Perms {
      */
     function &getPermissionById($cid)
     {
-        $object = $this->_datatree->getObjectById($cid, 'DataTreeObject_Permission');
+        if ($cid == DATATREE_ROOT) {
+            $object = &$this->newPermission(DATATREE_ROOT);
+        } else {
+            $object = &$this->_datatree->getObjectById($cid, 'DataTreeObject_Permission');
+        }
         return $object;
     }
 
@@ -138,8 +186,10 @@ class Perms_datatree extends Perms {
      *                           user.
      * @param string $creator    The user who created the object.
      *
-     * @return mixed  A bitmask of permissions the user has, false if there
-     *                are none.
+     * @return mixed  A bitmask of permissions, a permission value, or an array
+     *                of permission values the user has, depending on the
+     *                permission type and whether the permission value is
+     *                ambiguous. False if there is no such permsission.
      */
     function getPermissions($permission, $user = null, $creator = null)
     {
@@ -151,7 +201,7 @@ class Perms_datatree extends Perms {
             }
         }
 
-        if (is_null($user)) {
+        if ($user === null) {
             $user = Auth::getAuth();
         }
 
@@ -161,7 +211,7 @@ class Perms_datatree extends Perms {
         }
 
         // If $creator was specified, check creator permissions.
-        if (!is_null($creator)) {
+        if ($creator !== null) {
             // If the user is the creator of the event see if there
             // are creator permissions.
             if (strlen($user) && $user === $creator &&
@@ -184,16 +234,21 @@ class Perms_datatree extends Perms {
             $groups = &Group::singleton();
 
             $composite_perm = null;
+            $type = $permission->get('type');
             foreach ($permission->data['groups'] as $group => $perm) {
-                if ($groups->userIsInGroup($user, $groups->getGroupName($group))) {
-                    if (is_null($composite_perm)) {
-                        $composite_perm = 0;
+                if ($groups->userIsInGroup($user, $group)) {
+                    if ($composite_perm === null) {
+                        $composite_perm = $type == 'matrix' ? 0 : array();
+                    }
+                    if ($type == 'matrix') {
+                        $composite_perm |= $perm;
+                    } else {
+                        $composite_perm[] = $perm;
                     }
-                    $composite_perm |= $perm;
                 }
             }
 
-            if (!is_null($composite_perm)) {
+            if ($composite_perm !== null) {
                 return $composite_perm;
             }
         }
@@ -290,11 +345,34 @@ class DataTreeObject_Permission extends DataTreeObject {
      * The DataTreeObject_Permission constructor. Just makes sure to call the
      * parent constructor so that the perm's name is set properly.
      *
-     * @param string $name The name of the perm.
+     * @param string $name   The name of the perm.
+     * @param string $type   The permission type.
+     * @param array $params  A hash with any parameters that the permission
+     *                       type needs.
      */
-    function DataTreeObject_Permission($name)
+    function DataTreeObject_Permission($name, $type = 'matrix', $params = null)
     {
         parent::DataTreeObject($name);
+        $this->data['type'] = $type;
+        if (is_array($params)) {
+            $this->data['params'] = $params;
+        }
+    }
+
+    /**
+     * Gets one of the attributes of the object, or null if it isn't defined.
+     *
+     * @param string $attribute  The attribute to get.
+     *
+     * @return mixed  The value of the attribute, or null.
+     */
+    function get($attribute)
+    {
+        $value = parent::get($attribute);
+        if ($value === null && $attribute == 'type') {
+            $value = 'matrix';
+        }
+        return $value;
     }
 
     /**
@@ -305,20 +383,30 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function updatePermissions($perms)
     {
-        /* Array of permission types to iterate through. */
-        $perm_types = Perms::getPermsArray();
+        $type = $this->get('type');
+
+        if ($type == 'matrix') {
+            /* Array of permission types to iterate through. */
+            $perm_types = Perms::getPermsArray();
+        }
 
         foreach ($perms as $perm_class => $perm_values) {
             switch ($perm_class) {
             case 'default':
             case 'guest':
             case 'creator':
-                foreach ($perm_types as $val => $label) {
-                    if (!empty($perm_values[$val])) {
-                        $this->setPerm($perm_class, $val, false);
-                    } else {
-                        $this->unsetPerm($perm_class, $val, false);
+                if ($type == 'matrix') {
+                    foreach ($perm_types as $val => $label) {
+                        if (!empty($perm_values[$val])) {
+                            $this->setPerm($perm_class, $val, false);
+                        } else {
+                            $this->unsetPerm($perm_class, $val, false);
+                        }
                     }
+                } elseif (!empty($perm_values)) {
+                    $this->setPerm($perm_class, $perm_values, false);
+                } else {
+                    $this->unsetPerm($perm_class, null, false);
                 }
                 break;
 
@@ -328,7 +416,9 @@ class DataTreeObject_Permission extends DataTreeObject {
                 /* Figure out what names that are stored in this permission
                  * class have not been submitted for an update, ie. have been
                  * removed entirely. */
-                $current_names = isset($this->data[$permId['class']]) ? array_keys($this->data[$permId['class']]) : array();
+                $current_names = isset($this->data[$permId['class']])
+                    ? array_keys($this->data[$permId['class']])
+                    : array();
                 $updated_names = array_keys($perm_values);
                 $removed_names = array_diff($current_names, $updated_names);
 
@@ -338,7 +428,7 @@ class DataTreeObject_Permission extends DataTreeObject {
                 }
 
                 /* If nothing to actually update finish with this case. */
-                if (is_null($perm_values)) {
+                if ($perm_values === null) {
                     continue;
                 }
 
@@ -346,14 +436,21 @@ class DataTreeObject_Permission extends DataTreeObject {
                 foreach ($perm_values as $name => $name_values) {
                     $permId['name'] = $name;
 
-                    foreach ($perm_types as $val => $label) {
-                        if (!empty($name_values[$val])) {
-                            $this->setPerm($permId, $val, false);
-                        } else {
-                            $this->unsetPerm($permId, $val, false);
+                    if ($type == 'matrix') {
+                        foreach ($perm_types as $val => $label) {
+                            if (!empty($name_values[$val])) {
+                                $this->setPerm($permId, $val, false);
+                            } else {
+                                $this->unsetPerm($permId, $val, false);
+                            }
                         }
+                    } elseif (!empty($name_values)) {
+                        $this->setPerm($permId, $name_values, false);
+                    } else {
+                        $this->unsetPerm($permId, null, false);
                     }
                 }
+
                 break;
             }
         }
@@ -368,13 +465,15 @@ class DataTreeObject_Permission extends DataTreeObject {
             if (empty($permId['name'])) {
                 return;
             }
-            if (isset($this->data[$permId['class']][$permId['name']])) {
+            if ($this->get('type') == 'matrix' &&
+                isset($this->data[$permId['class']][$permId['name']])) {
                 $this->data[$permId['class']][$permId['name']] |= $permission;
             } else {
                 $this->data[$permId['class']][$permId['name']] = $permission;
             }
         } else {
-            if (isset($this->data[$permId])) {
+            if ($this->get('type') == 'matrix' &&
+                isset($this->data[$permId])) {
                 $this->data[$permId] |= $permission;
             } else {
                 $this->data[$permId] = $permission;
@@ -395,18 +494,32 @@ class DataTreeObject_Permission extends DataTreeObject {
             if (empty($permId['name'])) {
                 return;
             }
-            if (isset($this->data[$permId['class']][$permId['name']])) {
-                $this->data[$permId['class']][$permId['name']] &= ~$permission;
-                if (empty($this->data[$permId['class']][$permId['name']])) {
-                    unset($this->data[$permId['class']][$permId['name']]);
+            if ($this->get('type') == 'matrix') {
+                if (isset($this->data[$permId['class']][$permId['name']])) {
+                    $this->data[$permId['class']][$permId['name']] &= ~$permission;
+                    if (empty($this->data[$permId['class']][$permId['name']])) {
+                        unset($this->data[$permId['class']][$permId['name']]);
+                    }
+                    if ($update) {
+                        $this->save();
+                    }
                 }
+            } else {
+                unset($this->data[$permId['class']][$permId['name']]);
                 if ($update) {
                     $this->save();
                 }
             }
         } else {
-            if (isset($this->data[$permId])) {
-                $this->data[$permId] &= ~$permission;
+            if ($this->get('type') == 'matrix') {
+                if (isset($this->data[$permId])) {
+                    $this->data[$permId] &= ~$permission;
+                    if ($update) {
+                        $this->save();
+                    }
+                }
+            } else {
+                unset($this->data[$permId]);
                 if ($update) {
                     $this->save();
                 }
@@ -428,7 +541,8 @@ class DataTreeObject_Permission extends DataTreeObject {
         if (empty($user)) {
             return;
         }
-        if (isset($this->data['users'][$user])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['users'][$user])) {
             $this->data['users'][$user] |= $permission;
         } else {
             $this->data['users'][$user] = $permission;
@@ -448,7 +562,8 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function addGuestPermission($permission, $update = true)
     {
-        if (isset($this->data['guest'])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['guest'])) {
             $this->data['guest'] |= $permission;
         } else {
             $this->data['guest'] = $permission;
@@ -468,7 +583,8 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function addCreatorPermission($permission, $update = true)
     {
-        if (isset($this->data['creator'])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['creator'])) {
             $this->data['creator'] |= $permission;
         } else {
             $this->data['creator'] = $permission;
@@ -488,7 +604,8 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function addDefaultPermission($permission, $update = true)
     {
-        if (isset($this->data['default'])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['default'])) {
             $this->data['default'] |= $permission;
         } else {
             $this->data['default'] = $permission;
@@ -514,7 +631,8 @@ class DataTreeObject_Permission extends DataTreeObject {
             return;
         }
 
-        if (isset($this->data['groups'][$groupId])) {
+        if ($this->get('type') == 'matrix' &&
+            isset($this->data['groups'][$groupId])) {
             $this->data['groups'][$groupId] |= $permission;
         } else {
             $this->data['groups'][$groupId] = $permission;
@@ -536,17 +654,21 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function removeUserPermission($user, $permission, $update = true)
     {
-        if (empty($user)) {
+        if (empty($user) || !isset($this->data['users'][$user])) {
             return;
         }
-        if (isset($this->data['users'][$user])) {
+
+        if ($this->get('type') == 'matrix') {
             $this->data['users'][$user] &= ~$permission;
             if (empty($this->data['users'][$user])) {
                 unset($this->data['users'][$user]);
             }
-            if ($update) {
-                $this->save();
-            }
+        } else {
+            unset($this->data['users'][$user]);
+        }
+
+        if ($update) {
+            $this->save();
         }
     }
 
@@ -560,11 +682,20 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function removeGuestPermission($permission, $update = true)
     {
-        if (isset($this->data['guest'])) {
+        if (!isset($this->data['guest'])) {
+            return;
+        }
+
+        if ($this->get('type') == 'matrix') {
             $this->data['guest'] &= ~$permission;
             if ($update) {
                 $this->save();
             }
+        } else {
+            unset($this->data['guest']);
+            if ($update) {
+                $this->save();
+            }
         }
     }
 
@@ -578,11 +709,20 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function removeCreatorPermission($permission, $update = true)
     {
-        if (isset($this->data['creator'])) {
+        if (!isset($this->data['creator'])) {
+            return;
+        }
+
+        if ($this->get('type') == 'matrix') {
             $this->data['creator'] &= ~$permission;
             if ($update) {
                 $this->save();
             }
+        } else {
+            unset($this->data['creator']);
+            if ($update) {
+                $this->save();
+            }
         }
     }
 
@@ -596,11 +736,20 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function removeDefaultPermission($permission, $update = true)
     {
-        if (isset($this->data['default'])) {
+        if (!isset($this->data['default'])) {
+            return;
+        }
+
+        if ($this->get('type') == 'matrix') {
             $this->data['default'] &= ~$permission;
             if ($update) {
                 $this->save();
             }
+        } else {
+            unset($this->data['default']);
+            if ($update) {
+                $this->save();
+            }
         }
     }
 
@@ -616,11 +765,11 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function removeGroupPermission($groupId, $permission, $update = true)
     {
-        if (empty($groupId)) {
+        if (empty($groupId) || !isset($this->data['groups'][$groupId])) {
             return;
         }
 
-        if (isset($this->data['groups'][$groupId])) {
+        if ($this->get('type') == 'matrix') {
             $this->data['groups'][$groupId] &= ~$permission;
             if (empty($this->data['groups'][$groupId])) {
                 unset($this->data['groups'][$groupId]);
@@ -628,6 +777,11 @@ class DataTreeObject_Permission extends DataTreeObject {
             if ($update) {
                 $this->save();
             }
+        } else {
+            unset($this->data['groups'][$groupId]);
+            if ($update) {
+                $this->save();
+            }
         }
     }
 
@@ -702,7 +856,8 @@ class DataTreeObject_Permission extends DataTreeObject {
      */
     function getGroupPermissions($perm = null)
     {
-        if (!isset($this->data['groups']) || !is_array($this->data['groups'])) {
+        if (!isset($this->data['groups']) ||
+            !is_array($this->data['groups'])) {
             return array();
         } elseif (!$perm) {
             return $this->data['groups'];
