@@ -48,7 +48,7 @@ define('_PREF_DEFAULT', 8);
  *     'help'   => *Name of the entry in the XML help file*
  * );
  *
- * $Horde: framework/Prefs/Prefs.php,v 1.137.10.29 2007/01/02 13:54:35 jan Exp $
+ * $Horde: framework/Prefs/Prefs.php,v 1.137.10.30 2007/06/20 11:24:21 jan Exp $
  *
  * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
@@ -826,7 +826,7 @@ class Prefs {
             $params = Horde::getDriverConfig('prefs', $driver);
         }
 
-        $signature = serialize(array($driver, $user, $params));
+        $signature = serialize(array($driver, $user, $params, $caching));
         if (!isset($instances[$signature])) {
             $instances[$signature] = &Prefs::factory($driver, $scope, $user, $password, $params, $caching);
         }
