@@ -1,4 +1,10 @@
 <?php
+
+/**
+ * Horde_String class.
+ */
+require_once 'Horde/String.php';
+
 /**
  * Preferences storage implementation for PHP's LDAP extention.
  *
@@ -41,9 +47,9 @@
  * </pre>
  *
  *
- * $Horde: framework/Prefs/Prefs/ldap.php,v 1.85.10.11 2005/09/28 10:24:06 jan Exp $
+ * $Horde: framework/Prefs/Prefs/ldap.php,v 1.85.10.23 2007/05/10 16:29:11 jan Exp $
  *
- * Copyright 1999-2005 Jon Parise <jon@horde.org>
+ * Copyright 1999-2007 Jon Parise <jon@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -149,7 +155,7 @@ class Prefs_ldap extends Prefs {
             Horde::logMessage(
                 sprintf('Failed to open an LDAP connection to %s.',
                         $this->_params['hostspec']),
-                __FILE__, __LINE__);
+                __FILE__, __LINE__, PEAR_LOG_ERR);
             return false;
         }
 
@@ -160,54 +166,73 @@ class Prefs_ldap extends Prefs {
                 Horde::logMessage(
                     sprintf('Set LDAP protocol version to %d failed: [%d] %s',
                             $this->_params['version'],
-                            ldap_errno($this->_connection),
-                            ldap_error($this->_connection)),
-                            __FILE__, __LINE__);
+                            ldap_errno($conn),
+                            ldap_error($conn)),
+                    __FILE__, __LINE__, PEAR_LOG_ERR);
             }
         }
 
+        /* If necessary, bind to the LDAP server as the user with search
+         * permissions. */
+        if (!empty($this->_params['searchdn'])) {
+            $bind = @ldap_bind($conn, $this->_params['searchdn'],
+                               $this->_params['searchpass']);
+            if (!$bind) {
+                Horde::logMessage(sprintf('Bind to server %s:%d with DN %s failed: [%d] %s',
+                                          $this->_params['hostspec'],
+                                          $this->_params['port'],
+                                          $this->_params['searchdn'],
+                                          ldap_errno($conn),
+                                          ldap_error($conn)),
+                                  __FILE__, __LINE__, PEAR_LOG_ERR);
+                return false;
+            }
+	}
+
         /* Register our callback function to handle referrals. */
         if (function_exists('ldap_set_rebind_proc') &&
             !ldap_set_rebind_proc($conn, array($this, '_rebindProc'))) {
             Horde::logMessage(
                 sprintf('Set rebind proc failed: [%d] %s',
-                        ldap_errno($this->_connection),
-                        ldap_error($this->_connection)),
-                __FILE__, __LINE__);
+                        ldap_errno($conn),
+                        ldap_error($conn)),
+                __FILE__, __LINE__, PEAR_LOG_ERR);
             return false;
         }
 
-        /* Define the DN of the current user */
-        $this->_dn = sprintf('%s=%s,%s', $this->_params['uid'][0],
-                            $this->_user,
-                            $this->_params['basedn']);
-
-        /* And the DN of the authenticating user (may be the same as above) */
-        if (!empty($this->_params['rootdn'])) {
-            $bind_dn = $this->_params['rootdn'];
-        } else {
-            $bind_dn = sprintf('%s=%s,%s', $this->_params['uid'][0],
-                               $this->_params['username'],
-                               $this->_params['basedn']);
-        }
-
         /* Store the connection handle at the instance level. */
         $this->_connection = $conn;
         $this->_connected = true;
 
-        /* Bind to the LDAP server as the authenticating user. */
-        $bind = @ldap_bind($this->_connection, $bind_dn,
-                           $this->_params['password']);
-        if (!$bind) {
-            Horde::logMessage(
-                sprintf('Bind to server %s:%d with DN %s failed: [%d] %s',
-                        $this->_params['hostspec'],
-                        $this->_params['port'],
-                        $bind_dn,
-                        ldap_errno($this->_connection),
-                        ldap_error($this->_connection)),
-                __FILE__, __LINE__);
-            return false;
+        if (!$this->_params['fetchdn']) {
+            /* Define the DN of the current user. */
+            $this->_dn = sprintf('%s=%s,%s', $this->_params['uid'][0],
+                                 $this->_user,
+                                 $this->_params['basedn']);
+
+            /* And the DN of the authenticating user (may be the same as above). */
+            if (!empty($this->_params['rootdn'])) {
+                $bind_dn = $this->_params['rootdn'];
+            } else {
+                $bind_dn = sprintf('%s=%s,%s', $this->_params['uid'][0],
+                                   $this->_params['username'],
+                                   $this->_params['basedn']);
+            }
+
+            /* Bind to the LDAP server as the authenticating user. */
+            $bind = @ldap_bind($this->_connection, $bind_dn,
+                               $this->_params['password']);
+            if (!$bind) {
+                Horde::logMessage(
+                    sprintf('Bind to server %s:%d with DN %s failed: [%d] %s',
+                            $this->_params['hostspec'],
+                            $this->_params['port'],
+                            $bind_dn,
+                            ldap_errno($this->_connection),
+                            ldap_error($this->_connection)),
+                    __FILE__, __LINE__, PEAR_LOG_ERR);
+                return false;
+            }
         }
 
         /* Search for the user's full DN. */
@@ -216,8 +241,13 @@ class Prefs_ldap extends Prefs {
                                   $uid_field . '=' . $this->_user, array('dn'));
             if ($search) {
                 $result = ldap_get_entries($this->_connection, $search);
-                if ($result && !empty($result[0]['dn'])) {
+                if ($result && $result['count'] > 0) {
                     $this->_dn = $result[0]['dn'];
+                    if ($this->_params['fetchdn']) {
+                        // Re-bind as the user.
+                        $bind = @ldap_bind($this->_connection, $this->_dn,
+                                           $this->_params['password']);
+                    }
                     return true;
                 }
             }
@@ -227,7 +257,7 @@ class Prefs_ldap extends Prefs {
             sprintf('Failed to retrieve user\'s DN: [%d] %s',
                     ldap_errno($this->_connection),
                     ldap_error($this->_connection)),
-            __FILE__, __LINE__);
+            __FILE__, __LINE__, PEAR_LOG_ERR);
 
         return false;
     }
@@ -257,8 +287,6 @@ class Prefs_ldap extends Prefs {
      * @access private
      *
      * @return integer  1 on error, 0 on success.
-     *
-     * @since Horde 2.1
      */
     function _rebindProc($conn, $who)
     {
@@ -279,7 +307,7 @@ class Prefs_ldap extends Prefs {
         if (strpos($this->_params['hostspec'], $who) === false) {
             Horde::logMessage(
                 sprintf('Referral target %s for DN %s is not in the authorized server list!', $who, $bind_dn),
-                __FILE__, __LINE__);
+                __FILE__, __LINE__, PEAR_LOG_ERR);
             return 1;
         }
 
@@ -293,7 +321,7 @@ class Prefs_ldap extends Prefs {
                         $bind_dn,
                         ldap_errno($this->_connection),
                         ldap_error($this->_connection)),
-                __FILE__, __LINE__);
+                __FILE__, __LINE__, PEAR_LOG_ERR);
         }
 
         return 0;
@@ -336,7 +364,7 @@ class Prefs_ldap extends Prefs {
             }
         }
         if (!isset($result)) {
-            Horde::logMessage('Failed to connect to LDAP preferences server.', __FILE__, __LINE__);
+            Horde::logMessage('Failed to connect to LDAP preferences server.', __FILE__, __LINE__, PEAR_LOG_ERR);
         }
 
         /* ldap_get_entries() converts attribute indexes to lowercase. */
@@ -389,15 +417,10 @@ class Prefs_ldap extends Prefs {
                 }
             }
 
-            /* Make sure we know that we've loaded these
-             * preferences. */
-            $_SESSION['prefs_cache']['_filled']['horde'] = true;
-            $_SESSION['prefs_cache']['_filled'][$this->_scope] = true;
-
             /* Call hooks. */
             $this->_callHooks();
         } else {
-            Horde::logMessage('No preferences were retrieved.', __FILE__, __LINE__);
+            Horde::logMessage('No preferences were retrieved.', __FILE__, __LINE__, PEAR_LOG_ERR);
             return;
         }
 
@@ -486,7 +509,7 @@ class Prefs_ldap extends Prefs {
                sprintf('Unable to modify preferences: [%d] %s',
                         ldap_errno($this->_connection),
                         ldap_error($this->_connection)),
-                __FILE__, __LINE__);
+                __FILE__, __LINE__, PEAR_LOG_ERR);
             $updated = false;
         }
 
@@ -515,7 +538,7 @@ class Prefs_ldap extends Prefs {
         if ($search) {
             $result = ldap_get_entries($this->_connection, $search);
         } else {
-            Horde::logMessage('Failed to connect to LDAP preferences server.', __FILE__, __LINE__);
+            Horde::logMessage('Failed to connect to LDAP preferences server.', __FILE__, __LINE__, PEAR_LOG_ERR);
         }
 
         $attrs = array();
@@ -528,14 +551,14 @@ class Prefs_ldap extends Prefs {
                     sprintf('Unable to clear preferences: [%d] %s',
                             ldap_errno($this->_connection),
                             ldap_error($this->_connection)),
-                    __FILE__, __LINE__);
+                    __FILE__, __LINE__, PEAR_LOG_ERR);
             }
         } else {
             Horde::logMessage(
                 sprintf('Unable to clear preferences: [%d] %s',
                         ldap_errno($this->_connection),
                         ldap_error($this->_connection)),
-                __FILE__, __LINE__);
+                __FILE__, __LINE__, PEAR_LOG_ERR);
         }
 
         $this->cleanup(true);
