@@ -4,7 +4,6 @@
  *
  * Required parameters:<pre>
  *   'hostspec'  The hostname of the database server.
- *   'protocol'  The communication protocol ('tcp', 'unix', etc.).
  *   'username'  The username with which to connect to the database.
  *   'password'  The password associated with 'username'.
  *   'database'  The name of the database.
@@ -19,9 +18,9 @@
  * The table structure for the SessionHandler can be found in
  * horde/scripts/sql/horde_sessionhandler.oci8.sql.
  *
- * $Horde: framework/SessionHandler/SessionHandler/oci8.php,v 1.8.4.8 2005/06/26 04:28:31 chuck Exp $
+ * $Horde: framework/SessionHandler/SessionHandler/oci8.php,v 1.8.4.14 2007/03/14 09:45:37 jan Exp $
  *
- * Copyright 2003-2005 Liam Hoekenga <liamr@umich.edu>
+ * Copyright 2003-2007 Liam Hoekenga <liamr@umich.edu>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -72,38 +71,27 @@ class SessionHandler_oci8 extends SessionHandler {
     function read($id)
     {
         /* Make sure we have a valid database connection. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return '';
+        }
 
         $select_query = sprintf('SELECT session_data FROM %s WHERE session_id = %s FOR UPDATE',
                                 $this->_params['table'], $this->_quote($id));
 
-        /* Log the query at a DEBUG log level. */
         Horde::logMessage(sprintf('SQL Query by SessionHandler_oci8::read(): query = "%s"', $select_query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        /* Execute query */
         $select_statement = OCIParse($this->_db, $select_query);
-        OCIExecute($select_statement);
-        if (!OCIFetchInto($select_statement, $result)) {
-            $insert_query = sprintf('INSERT INTO %s (session_id, session_lastmodified, session_data) VALUES (%s, %s, EMPTY_BLOB()) RETURNING session_data INTO :blob',
-                                    $this->_params['table'],
-                                    $this->_quote($id),
-                                    $this->_quote(time()));
-            $insert_statement = OCIParse($this->_db, $insert_query);
-            $lob = OCINewDescriptor($this->_db);
-            OCIBindByName($insert_statement, ':blob', $lob, -1, SQLT_BLOB);
-            OCIExecute($insert_statement, OCI_DEFAULT);
-            if ($session_data) {
-                $lob->save($session_data);
-            }
-            $result = OCICommit($this->_db);
-            OCIFreeStatement($insert_statement);
-            OCIExecute($select_statement);
-            OCIFetchInto($select_statement, $result);
+        OCIExecute($select_statement, OCI_DEFAULT);
+        if (OCIFetchInto($select_statement, $result)) {
+            $value = $result[0]->load();
+        } else {
+            $value = '';
         }
-        $value = $result[0]->load();
+
         OCIFreeStatement($select_statement);
-        return($value);
+        return $value;
     }
 
     /**
@@ -117,30 +105,53 @@ class SessionHandler_oci8 extends SessionHandler {
     function write($id, $session_data)
     {
         /* Make sure we have a valid database connection. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return false;
+        }
 
-        /* Build the SQL query. */
-        $query = sprintf('UPDATE %s SET session_lastmodified = %s, session_data = EMPTY_BLOB() WHERE session_id = %s RETURNING session_data INTO :blob',
-                         $this->_params['table'],
-                         $this->_quote(time()),
-                         $this->_quote($id));
+        $select_query = sprintf('SELECT session_data FROM %s WHERE session_id = %s FOR UPDATE',
+                                $this->_params['table'], $this->_quote($id));
 
-        /* Log the query at a DEBUG log level. */
-        Horde::logMessage(sprintf('SQL Query by SessionHandler_oci8::write(): query = "%s"', $query),
+        Horde::logMessage(sprintf('SQL Query by SessionHandler_oci8::write(): query = "%s"', $select_query),
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
-        /* Execute query */
-        $statement = OCIParse($this->_db, $query);
-        $lob = OCINewDescriptor($this->_db);
-        OCIBindByName($statement, ':blob', $lob, -1, SQLT_BLOB);
-        OCIExecute($statement, OCI_DEFAULT);
-        if ($session_data) {
-            $lob->save($session_data);
-        }
-        $result = OCICommit($this->_db);
-        if (!$result) {
-            Horde::logMessage('Error writing session data', __FILE__, __LINE__, PEAR_LOG_ERR);
-            return false;
+        $select_statement = OCIParse($this->_db, $select_query);
+        OCIExecute($select_statement, OCI_DEFAULT);
+        if (OCIFetchInto($select_statement, $result)) {
+            /* Discard the existing LOB contents. */
+            if (!$result[0]->truncate()) {
+                OCIRollback($this->_db);
+                return false;
+            }
+
+            /* Save the session data. */
+            if ($result[0]->save($session_data)) {
+                OCICommit($this->_db);
+                OCIFreeStatement($select_statement);
+            } else {
+                OCIRollback($this->_db);
+                return false;
+            }
+        } else {
+            $insert_query = sprintf('INSERT INTO %s (session_id, session_lastmodified, session_data) VALUES (%s, %s, EMPTY_BLOB()) RETURNING session_data INTO :blob',
+                                    $this->_params['table'],
+                                    $this->_quote($id),
+                                    $this->_quote(time()));
+
+            Horde::logMessage(sprintf('SQL Query by SessionHandler_oci8::read(): query = "%s"', $insert_query),
+                              __FILE__, __LINE__, PEAR_LOG_DEBUG);
+
+            $insert_statement = OCIParse($this->_db, $insert_query);
+            $lob = OCINewDescriptor($this->_db);
+            OCIBindByName($insert_statement, ':blob', $lob, -1, SQLT_BLOB);
+            OCIExecute($insert_statement, OCI_DEFAULT);
+            if (!$lob->save($session_data)) {
+                OCIRollback($this->_db);
+                return false;
+            }
+            OCICommit($this->_db);
+            OCIFreeStatement($insert_statement);
         }
 
         return true;
@@ -157,11 +168,14 @@ class SessionHandler_oci8 extends SessionHandler {
     function destroy($id)
     {
         /* Make sure we have a valid database connection. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return false;
+        }
 
         /* Build the SQL query. */
-        $query = sprintf( 'DELETE FROM %s WHERE session_id = %s',
-                          $this->_params['table'], $this->_quote($id));
+        $query = sprintf('DELETE FROM %s WHERE session_id = %s',
+                         $this->_params['table'], $this->_quote($id));
 
         /* Log the query at a DEBUG log level. */
         Horde::logMessage(sprintf('SQL Query by SessionHandler_oci8::destroy(): query = "%s"', $query),
@@ -190,7 +204,10 @@ class SessionHandler_oci8 extends SessionHandler {
     function gc($maxlifetime = 1)
     {
         /* Make sure we have a valid database connection. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return false;
+        }
 
         /* Build the SQL query. */
         $query = sprintf('DELETE FROM %s WHERE session_lastmodified < %s',
@@ -221,7 +238,10 @@ class SessionHandler_oci8 extends SessionHandler {
     function getSessionIDs()
     {
         /* Make sure we have a valid database connection. */
-        $this->_connect();
+        if (is_a(($result = $this->_connect()), 'PEAR_Error')) {
+            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return false;
+        }
 
         /* Session timeout, don't rely on garbage collection */
         $timeout = time() - ini_get('session.gc_maxlifetime');
@@ -269,30 +289,39 @@ class SessionHandler_oci8 extends SessionHandler {
     function _connect()
     {
         if ($this->_connected) {
-            return;
+            return true;
         }
 
         Horde::assertDriverConfig($this->_params, 'sessionhandler',
-            array('hostspec', 'username', 'database', 'password'),
+            array('hostspec', 'username', 'password'),
             'session handler Oracle');
 
         if (!isset($this->_params['table'])) {
             $this->_params['table'] = 'horde_sessionhandler';
         }
 
-        if (empty($this->_params['persistent'])) {
-            $connect = 'OCILogon';
+        if (function_exists('oci_connect')) {
+            if (empty($this->_params['persistent'])) {
+                $connect = 'oci_connect';
+            } else {
+                $connect = 'oci_pconnect';
+            }
         } else {
-            $connect = 'OCIPLogon';
+            if (empty($this->_params['persistent'])) {
+                $connect = 'OCILogon';
+            } else {
+                $connect = 'OCIPLogon';
+            }
         }
 
         if (!is_resource($this->_db = @$connect($this->_params['username'],
                                                 $this->_params['password'],
                                                 $this->_params['hostspec']))) {
-            Horde::fatal(PEAR::raiseError('Could not connect to database for SQL SessionHandler.'), __FILE__, __LINE__);
+            return PEAR::raiseError('Could not connect to database for SQL SessionHandler.');
         }
 
         $this->_connected = true;
+        return true;
     }
 
 }
