@@ -7,11 +7,11 @@ require_once 'Horde/DataTree.php';
  * methods take care of any site-specific restrictions configured in in the
  * application's prefs.php and conf.php files.
  *
- * $Horde: framework/Share/Share.php,v 1.111.2.14 2005/09/23 07:16:12 jan Exp $
+ * $Horde: framework/Share/Share.php,v 1.111.2.23 2007/01/14 17:04:56 jan Exp $
  *
- * Copyright 2002-2005 Joel Vandal <jvandal@infoteck.qc.ca>
- * Copyright 2002-2005 Infoteck Internet <webmaster@infoteck.qc.ca>
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Joel Vandal <jvandal@infoteck.qc.ca>
+ * Copyright 2002-2007 Infoteck Internet <webmaster@infoteck.qc.ca>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -227,6 +227,9 @@ class Horde_Share {
         }
 
         $perm = &$GLOBALS['perms']->newPermission($share->getName());
+        if (is_a($perm, 'PEAR_Error')) {
+            return $perm;
+        }
 
         /* Give the owner full access */
         $perm->addUserPermission($share->get('owner'), PERMS_SHOW, false);
@@ -369,20 +372,22 @@ class Horde_Share {
      *
      * @param string  $userid      The userid of the user to check access for.
      * @param integer $perm        The level of permissions required.
-     * @param string  $owner       Restrict the shares counted to those owned
-     *                             by $owner.
+     * @param mixed   $attributes  Restrict the shares counted to those
+     *                             matching $attributes. An array of
+     *                             attribute/values pairs or a share owner
+     *                             username.
      * @param string  $parent      The parent share to start searching at.
      * @param boolean $allLevels   Return all levels, or just the direct
      *                             children of $parent? Defaults to all levels.
      *
      * @return integer  Number of shares the user has access to.
      */
-    function &countShares($userid, $perm = PERMS_SHOW, $owner = null,
-                          $parent = DATATREE_ROOT, $allLevels = true)
+    function countShares($userid, $perm = PERMS_SHOW, $attributes = null,
+                         $parent = DATATREE_ROOT, $allLevels = true)
     {
-        $key = serialize(array($this->_app, $userid, $perm, $owner, $parent, $allLevels, 'count'));
+        $key = serialize(array($this->_app, $userid, $perm, $attributes, $parent, $allLevels, 'count'));
         if (empty($this->_listcache[$key])) {
-            $criteria = $this->getShareCriteria($userid, $perm, $owner);
+            $criteria = $this->getShareCriteria($userid, $perm, $attributes);
             $this->_listcache[$key] = $this->_datatree->countByAttributes($criteria, $parent, $allLevels, 'id');
         }
 
@@ -392,25 +397,39 @@ class Horde_Share {
     /**
      * Returns an array of all shares that $userid has access to.
      *
-     * @param string  $userid     The userid of the user to check access for.
-     * @param integer $perm       The level of permissions required.
-     * @param string  $owner      Restrict the shares counted to those owned
-     *                            by $owner.
-     * @param string  $parent     The parent share to start searching at.
-     * @param boolean $allLevels  Return all levels, or just the direct
-     *                            children of $parent? Defaults to all levels.
-     * @param integer $from       The share to start listing at.
-     * @param integer $count      The number of shares to return.
+     * @param string  $userid       The userid of the user to check access for.
+     * @param integer $perm         The level of permissions required.
+     * @param mixed   $attributes   Restrict the shares counted to those
+     *                              matching $attributes. An array of
+     *                              attribute/values pairs or a share owner
+     *                              username.
+     * @param string  $parent       The parent share to start searching at.
+     * @param boolean $allLevels    Return all levels, or just the direct
+     *                              children of $parent? Defaults to all
+     *                              levels.
+     * @param integer $from         The share to start listing at.
+     * @param integer $count        The number of shares to return.
+     * @param string  $sortby_name  Attribute name to use for sorting.
+     * @param string  $sortby_key   Attribute key to use for sorting.
+     * @param integer $direction    Sort direction:
+     *                                0 - ascending
+     *                                1 - descending
      *
      * @return array  The shares the user has access to.
      */
-    function &listShares($userid, $perm = PERMS_SHOW, $owner = null,
-                         $parent = DATATREE_ROOT, $allLevels = true, $from = 0, $count = 0)
+    function &listShares($userid, $perm = PERMS_SHOW, $attributes = null,
+                         $parent = DATATREE_ROOT, $allLevels = true, $from = 0,
+                         $count = 0, $sortby_name = null, $sortby_key = null,
+                         $direction = 0)
     {
-        $key = serialize(array($this->_app, $userid, $perm, $owner, $parent, $allLevels, $from, $count));
+        $key = serialize(array($this->_app, $userid, $perm, $attributes,
+                               $parent, $allLevels, $from, $count,
+                               $sortby_name, $sortby_key, $direction));
         if (empty($this->_listcache[$key])) {
-            $criteria = $this->getShareCriteria($userid, $perm, $owner);
-            $sharelist = $this->_datatree->getByAttributes($criteria, $parent, $allLevels, 'id', $from, $count);
+            $criteria = $this->getShareCriteria($userid, $perm, $attributes);
+            $sharelist = $this->_datatree->getByAttributes(
+                $criteria, $parent, $allLevels, 'id', $from, $count,
+                $sortby_name, $sortby_key, $direction);
             if (is_a($sharelist, 'PEAR_Error') || !count($sharelist)) {
                 /* If we got back an error or an empty array, pass it back to
                  * the caller. */
@@ -430,7 +449,7 @@ class Horde_Share {
         }
 
         $result = Horde::callHook('_horde_hook_share_list', array($userid,
-                                  $perm, $owner, $this->_listcache[$key]),
+                                  $perm, $attributes, $this->_listcache[$key]),
                                   'horde', false);
         if (is_a($result, 'PEAR_Error')) {
             return $result;
@@ -473,14 +492,14 @@ class Horde_Share {
     /**
      * Returns an array of criteria for querying shares.
      *
-     * @param string  $userid  The userid of the user to check access for.
-     * @param integer $perm    The level of permissions required.
-     * @param string  $owner   Restrict the shares returned to those owned by
-     *                         $owner.
+     * @param string  $userid      The userid of the user to check access for.
+     * @param integer $perm        The level of permissions required.
+     * @param mixed   $attributes  Restrict the shares returned to those who
+     *                             have these attribute values.
      *
      * @return array  The criteria tree for fetching this user's shares.
      */
-    function getShareCriteria($userid, $perm = PERMS_SHOW, $owner = null)
+    function getShareCriteria($userid, $perm = PERMS_SHOW, $attributes = null)
     {
         if (!empty($userid)) {
             $criteria = array(
@@ -514,12 +533,12 @@ class Horde_Share {
             require_once 'Horde/Group.php';
             $group = &Group::singleton();
             $groups = $group->getGroupMemberships($userid, true);
-            if (!is_a($groups, 'PEAR_Error') && count($groups)) {
+            if (is_array($groups) && $groups) {
                 // (name == perm_groups and key in ($groups) and val & $perm)
                 $criteria['OR'][] = array(
                     'AND' => array(
                         array('field' => 'name', 'op' => '=', 'test' => 'perm_groups'),
-                        array('field' => 'key', 'op' => 'IN', 'test' => '(' . implode(', ', array_keys($groups)) . ')'),
+                        array('field' => 'key', 'op' => 'IN', 'test' => array_keys($groups)),
                         array('field' => 'value', 'op' => '&', 'test' => $perm)));
             }
         } else {
@@ -529,16 +548,28 @@ class Horde_Share {
                      array('field' => 'value', 'op' => '&', 'test' => $perm)));
         }
 
-        // Restrict to shares owned by $owner if $owner is non-empty
-        // (presumed to be a username).
-        if (!empty($owner)) {
+        if (is_array($attributes)) {
+            // Build attribute/key filter.
+            foreach ($attributes as $key => $value) {
+                $criteria = array(
+                    'AND' => array(
+                        $criteria,
+                        array(
+                            'JOIN' => array(
+                                'AND' => array(
+                                    array('field' => 'name', 'op' => '=', 'test' => $key),
+                                    array('field' => 'value', 'op' => '=', 'test' => $value))))));
+            }
+        } elseif (!empty($attributes)) {
+            // Restrict to shares owned by the user specified in the
+            // $attributes string.
             $criteria = array(
                 'AND' => array(
                     $criteria,
                     array(
                         'JOIN' => array(
                             array('field' => 'name', 'op' => '=', 'test' => 'owner'),
-                            array('field' => 'value', 'op' => '=', 'test' => $owner)))));
+                            array('field' => 'value', 'op' => '=', 'test' => $attributes)))));
         }
 
         return $criteria;
@@ -567,7 +598,6 @@ class Horde_Share {
     /**
      * Returns the Identity for a particular share owner.
      *
-     *
      * @param mixed $share  The share to fetch the Identity for - either the
      *                      string name, or the DataTreeObject_Share object.
      *
@@ -698,9 +728,9 @@ class DataTreeObject_Share extends DataTreeObject {
     /**
      * Checks to see if a user has a given permission.
      *
-     * @param string $userid   The userid of the user.
-     * @param integer $priv    A PERMS_* constant to test for.
-     * @param string $creator  The creator of the event.
+     * @param string $userid       The userid of the user.
+     * @param integer $permission  A PERMS_* constant to test for.
+     * @param string $creator      The creator of the event.
      *
      * @return boolean  Whether or not $userid has $permission.
      */
@@ -726,6 +756,23 @@ class DataTreeObject_Share extends DataTreeObject {
         $perm->removeUserPermission($userid, PERMS_READ, false);
         $perm->removeUserPermission($userid, PERMS_EDIT, false);
         $perm->removeUserPermission($userid, PERMS_DELETE, false);
+        return $this->setPermission($perm);
+    }
+
+    /**
+     * Removes a group from this share.
+     *
+     * @param integer $groupId  The group to remove.
+     */
+    function removeGroup($groupId)
+    {
+        /* Remove all $groupId's permissions. */
+        $perm = &$this->getPermission();
+        $perm->removeGroupPermission($groupId, PERMS_SHOW, false);
+        $perm->removeGroupPermission($groupId, PERMS_READ, false);
+        $perm->removeGroupPermission($groupId, PERMS_EDIT, false);
+        $perm->removeGroupPermission($groupId, PERMS_DELETE, false);
+        return $this->setPermission($perm);
     }
 
     /**
