@@ -1,10 +1,19 @@
 <?php
+/**
+ * @package Horde_iCalendar
+ */
+
+/**
+ * String package
+ */
+include_once 'Horde/String.php';
+
 /**
  * Class representing iCalendar files.
  *
- * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.14 2005/09/01 12:05:58 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.47 2007/11/23 21:26:35 jan Exp $
  *
- * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -45,6 +54,20 @@ class Horde_iCalendar {
      */
     var $_newline = "\r\n";
 
+    /**
+     * iCalendar format version (different behavior for 1.0 and 2.0
+     * especially with recurring events).
+     *
+     * @var string
+     */
+    var $_version;
+
+    function Horde_iCalendar($version = '2.0')
+    {
+        $this->_version = $version;
+        $this->setAttribute('VERSION', $version);
+    }
+
     /**
      * Return a reference to a new component.
      *
@@ -53,27 +76,34 @@ class Horde_iCalendar {
      *                                    will be associated with.
      *
      * @return object  Reference to a Horde_iCalendar_* object as specified.
+     *
+     * @static
      */
     function &newComponent($type, &$container)
     {
-        require_once 'Horde/String.php';
         $type = String::lower($type);
         $class = 'Horde_iCalendar_' . $type;
-        @include_once dirname(__FILE__) . '/iCalendar/' . $type . '.php';
+        if (!class_exists($class)) {
+            include_once 'Horde/iCalendar/' . $type . '.php';
+        }
         if (class_exists($class)) {
-            $component = &new $class();
+            $component = new $class();
             if ($container !== false) {
                 $component->_container = &$container;
+                // Use version of container, not default set by component
+                // constructor.
+                $component->_version = $container->_version;
             }
         } else {
             // Should return an dummy x-unknown type class here.
             $component = false;
         }
+
         return $component;
     }
 
     /**
-     * Set the value of an attribute.
+     * Sets the value of an attribute.
      *
      * @param string $name     The name of the attribute.
      * @param string $value    The value of the attribute.
@@ -85,20 +115,32 @@ class Horde_iCalendar {
      *                         comma/semicolon seperated lists of values.  If
      *                         not set use $value as single array element.
      */
-    function setAttribute($name, $value, $params = array(), $append = true, $values = false)
+    function setAttribute($name, $value, $params = array(), $append = true,
+                          $values = false)
     {
-        $found = $append;
+        // Make sure we update the internal format version if
+        // setAttribute('VERSION', ...) is called.
+        if ($name == 'VERSION') {
+            $this->_version = $value;
+            if ($this->_container !== false) {
+                $this->_container->_version = $value;
+            }
+        }
+
         if (!$values) {
             $values = array($value);
         }
-        $keys = array_keys($this->_attributes);
-        foreach ($keys as $key) {
-            if ($found) break;
-            if ($this->_attributes[$key]['name'] == String::upper($name)) {
-                $this->_attributes[$key]['params'] = $params;
-                $this->_attributes[$key]['value'] = $value;
-                $this->_attributes[$key]['values'] = $values;
-                $found = true;
+        $found = false;
+        if (!$append) {
+            $keys = array_keys($this->_attributes);
+            foreach ($keys as $key) {
+                if ($this->_attributes[$key]['name'] == String::upper($name)) {
+                    $this->_attributes[$key]['params'] = $params;
+                    $this->_attributes[$key]['value'] = $value;
+                    $this->_attributes[$key]['values'] = $values;
+                    $found = true;
+                    break;
+                }
             }
         }
 
@@ -121,7 +163,7 @@ class Horde_iCalendar {
      *                       this attribute.
      * @return boolean  True on success, false if no attribute $name exists.
      */
-    function setParameter($name, $params)
+    function setParameter($name, $params = array())
     {
         $keys = array_keys($this->_attributes);
         foreach ($keys as $key) {
@@ -274,6 +316,11 @@ class Horde_iCalendar {
         return $this->_components;
     }
 
+    function getType()
+    {
+        return 'vcalendar';
+    }
+
     /**
      * Return the classes (entry types) we have.
      *
@@ -323,18 +370,17 @@ class Horde_iCalendar {
     }
 
     /**
-     * Locates the first child component of the specified class, and
-     * returns a reference to this component.
+     * Locates the first child component of the specified class, and returns a
+     * reference to it.
      *
      * @param string $type  The type of component to find.
      *
-     * @return mixed (boolean) False if no subcomponent of the specified
-     *                         class exists.
-     *               (Horde_iCalendar_*) A reference to the requested component.
+     * @return boolean|Horde_iCalendar_*  False if no subcomponent of the
+     *                                    specified class exists or a reference
+     *                                    to the requested component.
      */
     function &findComponent($childclass)
     {
-        require_once 'Horde/String.php';
         $childclass = 'Horde_iCalendar_' . String::lower($childclass);
         $keys = array_keys($this->_components);
         foreach ($keys as $key) {
@@ -343,7 +389,42 @@ class Horde_iCalendar {
             }
         }
 
-        return false;
+        $component = false;
+        return $component;
+    }
+
+    /**
+     * Locates the first matching child component of the specified class, and
+     * returns a reference to it.
+     *
+     * @param string $childclass  The type of component to find.
+     * @param string $attribute   This attribute must be set in the component
+     *                            for it to match.
+     * @param string $value       Optional value that $attribute must match.
+     *
+     * @return boolean|Horde_iCalendar_*  False if no matching subcomponent of
+     *                                    the specified class exists, or a
+     *                                    reference to the requested component.
+     */
+    function &findComponentByAttribute($childclass, $attribute, $value = null)
+    {
+        $childclass = 'Horde_iCalendar_' . String::lower($childclass);
+        $keys = array_keys($this->_components);
+        foreach ($keys as $key) {
+            if (is_a($this->_components[$key], $childclass)) {
+                $attr = $this->_components[$key]->getAttribute($attribute);
+                if (is_a($attr, 'PEAR_Error')) {
+                    continue;
+                }
+                if ($value !== null && $value != $attr) {
+                    continue;
+                }
+                return $this->_components[$key];
+            }
+        }
+
+        $component = false;
+        return $component;
     }
 
     /**
@@ -356,13 +437,38 @@ class Horde_iCalendar {
         $this->_attributes = array();
     }
 
+    /**
+     * Checks if entry is vcalendar 1.0, vcard 2.1 or vnote 1.1.
+     *
+     * These 'old' formats are defined by www.imc.org. The 'new' (non-old)
+     * formats icalendar 2.0 and vcard 3.0 are defined in rfc2426 and rfc2445
+     * respectively.
+     *
+     * @since Horde 3.1.2
+     */
+    function isOldFormat()
+    {
+        if ($this->_container !== false) {
+            return $this->_container->isOldFormat();
+        }
+        if ($this->getType() == 'vcard') {
+            return $this->_version < 3 ? true : false;
+        }
+        if ($this->getType() == 'vNote') {
+            return $this->_version < 2 ? true : false;
+        }
+        if ($this->_version >= 2) {
+            return false;
+        }
+        return true;
+    }
+
     /**
      * Export as vCalendar format.
      */
     function exportvCalendar()
     {
         // Default values.
-        $requiredAttributes['VERSION'] = '2.0';
         $requiredAttributes['PRODID'] = '-//The Horde Project//Horde_iCalendar Library' . (defined('HORDE_VERSION') ? ', Horde ' . constant('HORDE_VERSION') : '') . '//EN';
         $requiredAttributes['METHOD'] = 'PUBLISH';
 
@@ -432,9 +538,32 @@ class Horde_iCalendar {
         // All subcomponents.
         $matches = null;
         if (preg_match_all('/BEGIN:([\W\w]*)(\r\n|\r|\n)([\W\w]*)END:\1(\r\n|\r|\n)/Ui', $vCal, $matches)) {
+            // vTimezone components are processed first. They are
+            // needed to process vEvents that may use a TZID.
             foreach ($matches[0] as $key => $data) {
-                $type = $matches[1][$key];
-                $component = &Horde_iCalendar::newComponent(trim($type), $this);
+                $type = trim($matches[1][$key]);
+                if ($type != 'VTIMEZONE') {
+                    continue;
+                }
+                $component = &Horde_iCalendar::newComponent($type, $this);
+                if ($component === false) {
+                    return PEAR::raiseError("Unable to create object for type $type");
+                }
+                $component->parsevCalendar($data);
+
+                $this->addComponent($component);
+
+                // Remove from the vCalendar data.
+                $vCal = str_replace($data, '', $vCal);
+            }
+
+            // Now process the non-vTimezone components.
+            foreach ($matches[0] as $key => $data) {
+                $type = trim($matches[1][$key]);
+                if ($type == 'VTIMEZONE') {
+                    continue;
+                }
+                $component = &Horde_iCalendar::newComponent($type, $this);
                 if ($component === false) {
                     return PEAR::raiseError("Unable to create object for type $type");
                 }
@@ -454,7 +583,7 @@ class Horde_iCalendar {
         //  BODY;ENCODING=QUOTED-PRINTABLE:=
         //  another=20line=
         //  last=20line
-        if (preg_match_all('/^([^:]+;\s*ENCODING=QUOTED-PRINTABLE(.*=\r?\n)+(.*[^=])?\r?\n)/mU', $vCal, $matches)) {
+        while (preg_match_all('/^([^:]+;\s*ENCODING=QUOTED-PRINTABLE(.*=\r?\n)+(.*[^=])?\r?\n)/mU', $vCal, $matches)) {
             foreach ($matches[1] as $s) {
                 $r = preg_replace('/=\r?\n/', '', $s);
                 $vCal = str_replace($s, $r, $vCal);
@@ -465,7 +594,7 @@ class Horde_iCalendar {
         if (preg_match_all('/(.*):([^\r\n]*)[\r\n]+/', $vCal, $matches)) {
             foreach ($matches[0] as $attribute) {
                 preg_match('/([^;^:]*)((;[^:]*)?):([^\r\n]*)[\r\n]*/', $attribute, $parts);
-                $tag = $parts[1];
+                $tag = String::upper($parts[1]);
                 $value = $parts[4];
                 $params = array();
 
@@ -474,13 +603,13 @@ class Horde_iCalendar {
                     preg_match_all('/;(([^;=]*)(=([^;]*))?)/', $parts[2], $param_parts);
                     foreach ($param_parts[2] as $key => $paramName) {
                         $paramValue = $param_parts[4][$key];
-                        $params[$paramName] = $paramValue;
+                        $params[String::upper($paramName)] = $paramValue;
                     }
                 }
 
                 // Charset and encoding handling.
                 if ((isset($params['ENCODING'])
-                     && $params['ENCODING'] == 'QUOTED-PRINTABLE')
+                     && String::upper($params['ENCODING']) == 'QUOTED-PRINTABLE')
                     || isset($params['QUOTED-PRINTABLE'])) {
 
                     $value = quoted_printable_decode($value);
@@ -490,23 +619,23 @@ class Horde_iCalendar {
                     } else {
                         $value = String::convertCharset($value, 'utf-8');
                     }
-                }
-
-                if (isset($params['CHARSET'])) {
+                } elseif (isset($params['CHARSET'])) {
                     $value = String::convertCharset($value, $params['CHARSET']);
                 } else {
-                    // As per RFC 2279, assume UTF8 if we don't have
-                    // an explicit charset parameter.
+                    // As per RFC 2279, assume UTF8 if we don't have an
+                    // explicit charset parameter.
                     $value = String::convertCharset($value, $charset);
                 }
 
+                // Get timezone info for date fields from $params.
+                $tzid = isset($params['TZID']) ? trim($params['TZID'], '\"') : false;
+
                 switch ($tag) {
                 // Date fields.
-                case 'DTSTAMP':
                 case 'COMPLETED':
                 case 'CREATED':
                 case 'LAST-MODIFIED':
-                    $this->setAttribute($tag, $this->_parseDateTime($value), $params);
+                    $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
                     break;
 
                 case 'BDAY':
@@ -515,19 +644,21 @@ class Horde_iCalendar {
 
                 case 'DTEND':
                 case 'DTSTART':
+                case 'DTSTAMP':
                 case 'DUE':
+                case 'AALARM':
                 case 'RECURRENCE-ID':
                     if (isset($params['VALUE']) && $params['VALUE'] == 'DATE') {
                         $this->setAttribute($tag, $this->_parseDate($value), $params);
                     } else {
-                        $this->setAttribute($tag, $this->_parseDateTime($value), $params);
+                        $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
                     }
                     break;
 
                 case 'TRIGGER':
                     if (isset($params['VALUE'])) {
                         if ($params['VALUE'] == 'DATE-TIME') {
-                            $this->setAttribute($tag, $this->_parseDateTime($value), $params);
+                            $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
                         } else {
                             $this->setAttribute($tag, $this->_parseDuration($value), $params);
                         }
@@ -581,8 +712,8 @@ class Horde_iCalendar {
                 // Geo fields.
                 case 'GEO':
                     $floats = explode(';', $value);
-                    $value['latitude'] = floatval($floats[0]);
-                    $value['longitude'] = floatval($floats[1]);
+                    $value = array('latitude' => floatval($floats[0]),
+                                   'longitude' => floatval($floats[1]));
                     $this->setAttribute($tag, $value, $params);
                     break;
 
@@ -592,18 +723,20 @@ class Horde_iCalendar {
                     $this->setAttribute($tag, trim($value), $params);
                     break;
 
-                // ADR an N are lists seperated by unescaped semi-colons.
+                // ADR, ORG and N are lists seperated by unescaped semicolons
+                // with a specific number of slots.
                 case 'ADR':
                 case 'N':
+                case 'ORG':
                     $value = trim($value);
-                    // As of rfc 2426 2.4.2 semi-colon, comma, and
-                    // colon must be escaped.
-                    $value = str_replace('\\n', $this->_newline, $value);
-                    $value = str_replace('\\,', ',', $value);
-                    $value = str_replace('\\:', ':', $value);
+                    // As of rfc 2426 2.4.2 semicolon, comma, and colon must
+                    // be escaped (comma is unescaped after splitting below).
+                    $value = str_replace(array('\\n', '\\N', '\\;', '\\:'),
+                                         array($this->_newline, $this->_newline, ';', ':'),
+                                         $value);
 
-                    // Split by unescaped semi-colons:
-                    $values = preg_split('/(?<!\\\\);/',$value);
+                    // Split by unescaped semicolons:
+                    $values = preg_split('/(?<!\\\\);/', $value);
                     $value = str_replace('\\;', ';', $value);
                     $values = str_replace('\\;', ';', $values);
                     $this->setAttribute($tag, trim($value), $params, true, $values);
@@ -611,19 +744,32 @@ class Horde_iCalendar {
 
                 // String fields.
                 default:
-                    $value = trim($value);
-                    // As of rfc 2426 2.4.2 semi-colon, comma, and
-                    // colon must be escaped.
-                    $value = str_replace('\\n', $this->_newline, $value);
-                    $value = str_replace('\\;', ';', $value);
-                    $value = str_replace('\\:', ':', $value);
+                    if ($this->isOldFormat()) {
+                        // vCalendar 1.0 and vcCard 2.1 only escape
+                        // semicolons and use unescaped semicolons to
+                        // create lists.
+                        $value = trim($value);
+                        // Split by unescaped semicolons:
+                        $values = preg_split('/(?<!\\\\);/', $value);
+                        $value = str_replace('\\;', ';', $value);
+                        $values = str_replace('\\;', ';', $values);
+                        $this->setAttribute($tag, trim($value), $params, true, $values);
+                    } else {
+                        $value = trim($value);
+                        // As of rfc 2426 2.4.2 semicolon, comma, and
+                        // colon must be escaped (comma is unescaped after
+                        // splitting below).
+                        $value = str_replace(array('\\n', '\\N', '\\;', '\\:', '\\\\'),
+                                             array($this->_newline, $this->_newline, ';', ':', '\\'),
+                                             $value);
 
-                    // Split by unescaped commas:
-                    $values = preg_split('/(?<!\\\\),/',$value);
-                    $value = str_replace('\\,', ',', $value);
-                    $values = str_replace('\\,', ',', $values);
+                        // Split by unescaped commas:
+                        $values = preg_split('/(?<!\\\\),/', $value);
+                        $value = str_replace('\\,', ',', $value);
+                        $values = str_replace('\\,', ',', $values);
 
-                    $this->setAttribute($tag, trim($value), $params, true, $values);
+                        $this->setAttribute($tag, trim($value), $params, true, $values);
+                    }
                     break;
                 }
             }
@@ -641,14 +787,15 @@ class Horde_iCalendar {
      */
     function _exportvData($base = 'VCALENDAR')
     {
-        $result  = 'BEGIN:' . String::upper($base) . $this->_newline;
+        $result = 'BEGIN:' . String::upper($base) . $this->_newline;
 
-        // Ensure that version is the first attribute.
-        $v = $this->getAttributeDefault('VERSION', false);
-        if ($v) {
-            $result .= 'VERSION:' . $v . $this->_newline;
+        // VERSION is not allowed for entries enclosed in VCALENDAR/ICALENDAR,
+        // as it is part of the enclosing VCALENDAR/ICALENDAR. See rfc2445
+        if ($base !== 'VEVENT' && $base !== 'VTODO' && $base !== 'VALARM' &&
+            $base !== 'VJOURNAL' && $base !== 'VFREEBUSY') {
+            // Ensure that version is the first attribute.
+            $result .= 'VERSION:' . $this->_version . $this->_newline;
         }
-
         foreach ($this->_attributes as $attribute) {
             $name = $attribute['name'];
             if ($name == 'VERSION') {
@@ -656,19 +803,31 @@ class Horde_iCalendar {
                 continue;
             }
 
-            $params = $attribute['params'];
             $params_str = '';
-
-            if (count($params)) {
+            $params = $attribute['params'];
+            if ($params) {
                 foreach ($params as $param_name => $param_value) {
-                    $params_str .= ";$param_name=$param_value";
+                    /* Skip CHARSET for iCalendar 2.0 data, not allowed. */
+                    if ($param_name == 'CHARSET' && !$this->isOldFormat()) {
+                        continue;
+                    }
+                    /* Skip VALUE=DATE for vCalendar 1.0 data, not allowed. */
+                    if ($this->isOldFormat() &&
+                         $param_name == 'VALUE' && $param_value == 'DATE') {
+                        continue;
+                    }
+
+                    if ($param_value === null) {
+                        $params_str .= ";$param_name";
+                    } else {
+                        $params_str .= ";$param_name=$param_value";
+                    }
                 }
             }
 
             $value = $attribute['value'];
             switch ($name) {
             // Date fields.
-            case 'DTSTAMP':
             case 'COMPLETED':
             case 'CREATED':
             case 'DCREATED':
@@ -678,11 +837,13 @@ class Horde_iCalendar {
 
             case 'DTEND':
             case 'DTSTART':
+            case 'DTSTAMP':
             case 'DUE':
+            case 'AALARM':
             case 'RECURRENCE-ID':
                 if (isset($params['VALUE'])) {
                     if ($params['VALUE'] == 'DATE') {
-                        $value = $this->_exportDate($value);
+                        $value = $this->_exportDate($value, $name == 'DTEND' ? '235959' : '000000');
                     } else {
                         $value = $this->_exportDateTime($value);
                     }
@@ -698,7 +859,7 @@ class Horde_iCalendar {
                 foreach ($value as $date) {
                     if (isset($params['VALUE'])) {
                         if ($params['VALUE'] == 'DATE') {
-                            $dates[] = $this->_exportDate($date);
+                            $dates[] = $this->_exportDate($date, '000000');
                         } elseif ($params['VALUE'] == 'PERIOD') {
                             $dates[] = $this->_exportPeriod($date);
                         } else {
@@ -754,22 +915,81 @@ class Horde_iCalendar {
 
             // Geo fields.
             case 'GEO':
-                $value = $value['latitude'] . ',' . $value['longitude'];
+                $value = $value['latitude'] . ';' . $value['longitude'];
                 break;
 
             // Recurrence fields.
             case 'EXRULE':
             case 'RRULE':
+                break;
 
             default:
+                if ($this->isOldFormat()) {
+                    if (is_array($attribute['values']) &&
+                        count($attribute['values']) > 1) {
+                        $values = $attribute['values'];
+                        if ($name == 'N' || $name == 'ADR' || $name == 'ORG') {
+                            $glue = ';';
+                        } else {
+                            $glue = ',';
+                        }
+                        $values = str_replace(';', '\\;', $values);
+                        $value = implode($glue, $values);
+                    } else {
+                        /* vcard 2.1 and vcalendar 1.0 escape only
+                         * semicolons */
+                        $value = str_replace(';', '\\;', $value);
+                    }
+                    // Text containing newlines or ASCII >= 127 must be BASE64
+                    // or QUOTED-PRINTABLE encoded. Currently we use
+                    // QUOTED-PRINTABLE as default.
+                    // FIXME: deal with base64 encodings!
+                    if (preg_match("/[^\x20-\x7F\r\n]/", $value) &&
+                        empty($params['ENCODING']))  {
+                        $params['ENCODING'] = 'QUOTED-PRINTABLE';
+                        $params_str .= ';ENCODING=QUOTED-PRINTABLE';
+                        // Add CHARSET as well. At least the synthesis client
+                        // gets confused otherwise
+                        if (empty($params['CHARSET'])) {
+                            $params['CHARSET'] = NLS::getCharset();
+                            $params_str .= ';CHARSET=' . $params['CHARSET'];
+                        }
+                    }
+                } else {
+                    if (is_array($attribute['values']) &&
+                        count($attribute['values']) > 1) {
+                        $values = $attribute['values'];
+                        if ($name == 'N' || $name == 'ADR' || $name == 'ORG') {
+                            $glue = ';';
+                        } else {
+                            $glue = ',';
+                        }
+                        // As of rfc 2426 2.5 semicolon and comma must be
+                        // escaped.
+                        $values = str_replace(array(';', ',', '\\'),
+                                              array('\\;', '\\,', '\\\\'),
+                                              $values);
+                        $value = implode($glue, $values);
+                    } else {
+                        // As of rfc 2426 2.5 semicolon and comma must be
+                        // escaped.
+                        $value = str_replace(array(';', ',', '\\'),
+                                             array('\\;', '\\,', '\\\\'),
+                                             $value);
+                    }
+                }
                 break;
             }
 
             if (!empty($params['ENCODING']) &&
-                $params['ENCODING'] == 'QUOTED-PRINTABLE' && strlen(trim($value)) > 0) {
+                $params['ENCODING'] == 'QUOTED-PRINTABLE' &&
+                strlen(trim($value))) {
+                /* quotedPrintableEncode does not escape CRLFs, but strange
+                 * enough single LFs. so convert everything to LF only and
+                 * replace afterwards. */
                 $value = str_replace("\r", '', $value);
                 $result .= "$name$params_str:=" . $this->_newline
-                    . $this->_quotedPrintableEncode($value)
+                    . str_replace('=0A', '=0D=0A', $this->_quotedPrintableEncode($value))
                     . $this->_newline;
             } else {
                 $attr_string = "$name$params_str:$value";
@@ -850,33 +1070,97 @@ class Horde_iCalendar {
     }
 
     /**
-     * Parse a DateTime field into a unix timestamp.
+     * Grok the TZID and return an offset in seconds from UTC for this
+     * date and time.
      */
-    function _parseDateTime($text)
+    function _parseTZID($date, $time, $tzid)
+    {
+        $vtimezone = $this->_container->findComponentByAttribute('vtimezone', 'TZID', $tzid);
+        if (!$vtimezone) {
+            return false;
+        }
+
+        $change_times = array();
+        foreach ($vtimezone->getComponents() as $o) {
+            $t = $vtimezone->parseChild($o, $date['year']);
+            if ($t !== false) {
+                $change_times[] = $t;
+            }
+        }
+
+        if (!$change_times) {
+            return false;
+        }
+
+        sort($change_times);
+
+        // Time is arbitrarily based on UTC for comparison.
+        $t = @gmmktime($time['hour'], $time['minute'], $time['second'],
+                       $date['month'], $date['mday'], $date['year']);
+
+        if ($t < $change_times[0]['time']) {
+            return $change_times[0]['from'];
+        }
+
+        for ($i = 0, $n = count($change_times); $i < $n - 1; $i++) {
+            if (($t >= $change_times[$i]['time']) &&
+                ($t < $change_times[$i + 1]['time'])) {
+                return $change_times[$i]['to'];
+            }
+        }
+
+        if ($t >= $change_times[$n - 1]['time']) {
+            return $change_times[$n - 1]['to'];
+        }
+
+        return false;
+    }
+
+    /**
+     * Parses a DateTime field and returns a unix timestamp. If the
+     * field cannot be parsed then the original text is returned
+     * unmodified.
+     *
+     * @todo This function should be moved to Horde_Date and made public.
+     */
+    function _parseDateTime($text, $tzid = false)
     {
         $dateParts = explode('T', $text);
         if (count($dateParts) != 2 && !empty($text)) {
             // Not a datetime field but may be just a date field.
-            if (!$date = $this->_parseDate($text)) {
-                return $date;
+            if (!preg_match('/^(\d{4})-?(\d{2})-?(\d{2})$/', $text, $match)) {
+                // Or not
+                return $text;
             }
-            return @gmmktime(0, 0, 0, $date['month'], $date['mday'], $date['year']);
+            $newtext = $text.'T000000';
+            $dateParts = explode('T', $newtext);
         }
 
-        if (!$date = $this->_parseDate($dateParts[0])) {
-            return $date;
+        if (!$date = Horde_iCalendar::_parseDate($dateParts[0])) {
+            return $text;
         }
-        if (!$time = $this->_parseTime($dateParts[1])) {
-            return $time;
+        if (!$time = Horde_iCalendar::_parseTime($dateParts[1])) {
+            return $text;
         }
 
-        if ($time['zone'] == 'UTC') {
-            return @gmmktime($time['hour'], $time['minute'], $time['second'],
-                             $date['month'], $date['mday'], $date['year']);
+        // Get timezone info for date fields from $tzid and container.
+        $tzoffset = ($time['zone'] == 'Local' && $tzid && is_a($this->_container, 'Horde_iCalendar')) ?
+            $this->_parseTZID($date, $time, $tzid) : false;
+        if ($time['zone'] == 'UTC' || $tzoffset !== false) {
+            $result = @gmmktime($time['hour'], $time['minute'], $time['second'],
+                                $date['month'], $date['mday'], $date['year']);
+            if ($tzoffset) {
+                $result -= $tzoffset;
+            }
         } else {
-            return @mktime($time['hour'], $time['minute'], $time['second'],
-                           $date['month'], $date['mday'], $date['year']);
+            // We don't know the timezone so assume local timezone.
+            // FIXME: shouldn't this be based on the user's timezone
+            // preference rather than the server's timezone?
+            $result = @mktime($time['hour'], $time['minute'], $time['second'],
+                              $date['month'], $date['mday'], $date['year']);
         }
+
+        return ($result !== false) ? $result : $text;
     }
 
     /**
@@ -886,8 +1170,8 @@ class Horde_iCalendar {
     {
         $temp = array();
         if (!is_object($value) && !is_array($value)) {
-            $TZOffset  = 3600 * substr(date('O'), 0, 3);
-            $TZOffset += 60 * substr(date('O'), 3, 2);
+            $tz = date('O', $value);
+            $TZOffset = (3600 * substr($tz, 0, 3)) + (60 * substr(date('O', $value), 3, 2));
             $value -= $TZOffset;
 
             $temp['zone']   = 'UTC';
@@ -898,44 +1182,17 @@ class Horde_iCalendar {
             $temp['minute'] = date('i', $value);
             $temp['second'] = date('s', $value);
         } else {
-            $dateOb = (object)$value;
-
-            // Minutes.
-            $TZOffset = substr(date('O'), 3, 2);
-            $thisMin = $dateOb->min - $TZOffset;
-
-            // Hours.
-            $TZOffset = substr(date('O'), 0, 3);
-            $thisHour = $dateOb->hour - $TZOffset;
-
-            if ($thisMin < 0) {
-                $thisHour -= 1;
-                $thisMin += 60;
-            }
-
-            if ($thisHour < 0) {
-                require_once 'Date/Calc.php';
-                $prevday = Date_Calc::prevDay($dateOb->mday, $dateOb->month, $dateOb->year);
-                $dateOb->mday  = substr($prevday, 6, 2);
-                $dateOb->month = substr($prevday, 4, 2);
-                $dateOb->year  = substr($prevday, 0, 4);
-                $thisHour += 24;
-            }
-
-            $temp['zone']   = 'UTC';
-            $temp['year']   = $dateOb->year;
-            $temp['month']  = $dateOb->month;
-            $temp['mday']   = $dateOb->mday;
-            $temp['hour']   = $thisHour;
-            $temp['minute'] = $dateOb->min;
-            $temp['second'] = $dateOb->sec;
+            $dateOb = new Horde_Date($value);
+            return Horde_iCalendar::_exportDateTime($dateOb->timestamp());
         }
 
         return Horde_iCalendar::_exportDate($temp) . 'T' . Horde_iCalendar::_exportTime($temp);
     }
 
     /**
-     * Parse a Time field.
+     * Parses a Time field.
+     *
+     * @static
      */
     function _parseTime($text)
     {
@@ -955,7 +1212,7 @@ class Horde_iCalendar {
     }
 
     /**
-     * Export a Time field.
+     * Exports a Time field.
      */
     function _exportTime($value)
     {
@@ -968,7 +1225,9 @@ class Horde_iCalendar {
     }
 
     /**
-     * Parse a Date field.
+     * Parses a Date field.
+     *
+     * @static
      */
     function _parseDate($text)
     {
@@ -987,14 +1246,23 @@ class Horde_iCalendar {
     }
 
     /**
-     * Export a Date field.
+     * Exports a date field.
+     *
+     * @param object|array $value  Date object or hash.
+     * @param string $autoconvert  If set, use this as time part to export the
+     *                             date as datetime when exporting to Vcalendar
+     *                             1.0. Examples: '000000' or '235959'
      */
-    function _exportDate($value)
+    function _exportDate($value, $autoconvert = false)
     {
         if (is_object($value)) {
             $value = array('year' => $value->year, 'month' => $value->month, 'mday' => $value->mday);
         }
-        return sprintf('%04d%02d%02d', $value['year'], $value['month'], $value['mday']);
+        if ($autoconvert !== false && $this->isOldFormat()) {
+            return sprintf('%04d%02d%02dT%s', $value['year'], $value['month'], $value['mday'], $autoconvert);
+        } else {
+            return sprintf('%04d%02d%02d', $value['year'], $value['month'], $value['mday']);
+        }
     }
 
     /**
