@@ -3,12 +3,12 @@
  * Net_SMS_sms2email_http Class implements the HTTP API for accessing the
  * sms2email (www.sms2email.com) SMS gateway.
  *
- * Copyright 2003-2006 Marko Djukic <marko@oblo.com>
+ * Copyright 2003-2007 Marko Djukic <marko@oblo.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * $Horde: framework/Net_SMS/SMS/sms2email_http.php,v 1.23.10.10 2006/01/01 21:28:29 jan Exp $
+ * $Horde: framework/Net_SMS/SMS/sms2email_http.php,v 1.23.10.12 2007/01/02 13:54:29 jan Exp $
  *
  * @author Marko Djukic <marko@oblo.com>
  * @package Net_SMS
@@ -46,12 +46,12 @@ class Net_SMS_sms2email_http extends Net_SMS {
      *
      * @param array $message  The array containing the message and its send
      *                        parameters.
-     * @param string $to      The destination string.
+     * @param array $to       The recipients.
      *
      * @return array  An array with the success status and additional
      *                information.
      */
-    function _send(&$message, $to)
+    function _send($message, $to)
     {
         /* Set up the sending url. */
         $url = sprintf('postmsg.php?username=%s&password=%s&message=%s',
