@@ -55,16 +55,15 @@ define('VFS_FLAG_WRITE', 2);
  *    mssql.textsize = 0 ; zero to pass through
  * </code>
  *
- * $Horde: framework/VFS/VFS/musql.php,v 1.30.4.10 2005/07/03 04:52:43 selsky Exp $
+ * $Horde: framework/VFS/VFS/musql.php,v 1.30.4.17 2007/01/02 13:54:48 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @since   Horde 2.2
  * @package VFS
  */
 class VFS_musql extends VFS_sql {
@@ -213,6 +212,18 @@ class VFS_musql extends VFS_sql {
             return PEAR::raiseError(_("Unable to rename VFS file."));
         }
 
+        if (strpos($newpath, '/') === false) {
+            $parent = '';
+            $path = $newpath;
+        } else {
+            list($parent, $path) = explode('/', $newpath, 2);
+        }
+        if (!$this->isFolder($parent, $path)) {
+            if (is_a($result = $this->autocreatePath($newpath), 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
         /* There may be one or more files with the same name but the user may
          * not have read access to them, so doesn't see them. So we have to
          * rename the one they have access to. */
@@ -361,17 +372,17 @@ class VFS_musql extends VFS_sql {
             return $conn;
         }
 
-        $files = array();
-        $fileList = array();
-
-        $fileList = $this->_db->getAll(sprintf('SELECT vfs_name, vfs_type, vfs_modified, vfs_owner, vfs_perms, vfs_data FROM %s
+        $length_op = $this->_getFileSizeOp();
+        $fileList = $this->_db->getAll(sprintf('SELECT vfs_name, vfs_type, vfs_modified, vfs_owner, vfs_perms, %s(vfs_data) FROM %s
                                                WHERE vfs_path = ? AND (vfs_owner = ? or vfs_perms && ?)',
+                                               $length_op,
                                                $this->_params['table']),
                                        array($path, $this->_params['user'], VFS_FLAG_READ));
         if (is_a($fileList, 'PEAR_Error')) {
             return $fileList;
         }
 
+        $files = array();
         foreach ($fileList as $line) {
             // Filter out dotfiles if they aren't wanted.
             if (!$dotfiles && substr($line[0], 0, 1) == '.') {
@@ -389,7 +400,7 @@ class VFS_musql extends VFS_sql {
                     $file['type'] = VFS::strtolower($name[count($name) - 1]);
                 }
 
-                $file['size'] = VFS::strlen($line[5]);
+                $file['size'] = $line[5];
             } elseif ($line[1] == VFS_FOLDER) {
                 $file['type'] = '**dir';
                 $file['size'] = -1;
@@ -404,7 +415,7 @@ class VFS_musql extends VFS_sql {
             $file['perms'] .= ($line[4] & VFS_FLAG_READ) ? 'r' : '-';
             $file['perms'] .= ($line[4] & VFS_FLAG_WRITE) ? 'w' : '-';
             $file['perms'] .= '-';
-            $file['group'] = '-';
+            $file['group'] = '';
 
             // filtering
             if ($this->_filterMatch($filter, $file['name'])) {
