@@ -46,7 +46,7 @@ define('VFS_FOLDER', 2);
  *    mssql.textsize = 0 ; zero to pass through
  * </pre>
  *
- * $Horde: framework/VFS/VFS/sql.php,v 1.85.4.29 2007/01/02 13:54:48 jan Exp $
+ * $Horde: framework/VFS/VFS/sql.php,v 1.85.4.30 2007/06/05 23:37:48 slusarz Exp $
  *
  * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
  *
@@ -115,7 +115,7 @@ class VFS_sql extends VFS {
             return $conn;
         }
 
-        $where = (is_null($path)) ? null : sprintf('WHERE vfs_path = %s', ((!strlen($path)) ? '""' : $this->_db->quote($this->_convertPath($path))));
+        $where = (is_null($path)) ? null : sprintf('WHERE vfs_path LIKE %s', ((!strlen($path)) ? '""' : $this->_db->quote($this->_convertPath($path) . '%')));
         $length_op = $this->_getFileSizeOp();
         $sql = sprintf(
             'SELECT SUM(%s(vfs_data)) FROM %s %s',
