@@ -1,4 +1,4 @@
--- $Horde: horde/scripts/sql/create.mysql.sql,v 1.4.6.9 2006/07/05 15:53:10 jan Exp $
+-- $Horde: horde/scripts/sql/create.mysql.sql,v 1.4.6.10 2007/09/27 22:36:49 jan Exp $
 --
 -- If you are installing Horde for the first time, you can simply
 -- direct this file to mysql as STDIN:
@@ -58,8 +58,6 @@ CREATE TABLE horde_users (
     PRIMARY KEY (user_uid)
 );
 
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_users TO horde@localhost;
-
 CREATE TABLE horde_prefs (
     pref_uid        VARCHAR(200) NOT NULL,
     pref_scope      VARCHAR(16) NOT NULL DEFAULT '',
@@ -72,8 +70,6 @@ CREATE TABLE horde_prefs (
 CREATE INDEX pref_uid_idx ON horde_prefs (pref_uid);
 CREATE INDEX pref_scope_idx ON horde_prefs (pref_scope);
 
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_prefs TO horde@localhost;
-
 CREATE TABLE horde_datatree (
        datatree_id INT NOT NULL,
        group_uid VARCHAR(255) NOT NULL,
@@ -103,9 +99,6 @@ CREATE INDEX datatree_attribute_idx ON horde_datatree_attributes (datatree_id);
 CREATE INDEX datatree_attribute_name_idx ON horde_datatree_attributes (attribute_name);
 CREATE INDEX datatree_attribute_key_idx ON horde_datatree_attributes (attribute_key);
 
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_datatree TO horde@localhost;
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_datatree_attributes TO horde@localhost;
-
 CREATE TABLE horde_tokens (
     token_address    VARCHAR(100) NOT NULL,
     token_id         VARCHAR(32) NOT NULL,
@@ -114,8 +107,6 @@ CREATE TABLE horde_tokens (
     PRIMARY KEY (token_address, token_id)
 );
 
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_tokens TO horde@localhost;
-
 CREATE TABLE horde_vfs (
     vfs_id        BIGINT NOT NULL,
     vfs_type      SMALLINT NOT NULL,
@@ -131,8 +122,6 @@ CREATE TABLE horde_vfs (
 CREATE INDEX vfs_path_idx ON horde_vfs (vfs_path);
 CREATE INDEX vfs_name_idx ON horde_vfs (vfs_name);
 
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_vfs TO horde@localhost;
-
 CREATE TABLE horde_histories (
     history_id       BIGINT NOT NULL,
     object_uid       VARCHAR(255) NOT NULL,
@@ -149,8 +138,6 @@ CREATE INDEX history_action_idx ON horde_histories (history_action);
 CREATE INDEX history_ts_idx ON horde_histories (history_ts);
 CREATE INDEX history_uid_idx ON horde_histories (object_uid);
 
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_histories TO horde@localhost;
-
 CREATE TABLE horde_sessionhandler (
     session_id             VARCHAR(32) NOT NULL,
     session_lastmodified   INT NOT NULL,
@@ -159,8 +146,4 @@ CREATE TABLE horde_sessionhandler (
     PRIMARY KEY (session_id)
 ) ENGINE = InnoDB;
 
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_sessionhandler TO horde@localhost;
-
-FLUSH PRIVILEGES;
-
 -- Done!
