@@ -4,7 +4,7 @@ set sqlblanklines on;
 /**
  * Oracle Table Creation Scripts.
  * 
- * $Horde: horde/scripts/sql/create.oci8.sql,v 1.4.8.6 2005/07/13 12:50:35 jan Exp $
+ * $Horde: horde/scripts/sql/create.oci8.sql,v 1.4.8.10 2006/06/29 16:29:03 jan Exp $
  * 
  * @author Miguel Ward <mward@aluar.com.ar>
  * 
@@ -49,8 +49,10 @@ rem conn horde/&horde_password@database
  */
 
 CREATE TABLE horde_users (
-    user_uid    VARCHAR2(255) NOT NULL,
-    user_pass   VARCHAR2(255) NOT NULL,
+    user_uid                    VARCHAR2(255) NOT NULL,
+    user_pass                   VARCHAR2(255) NOT NULL,
+    user_soft_expiration_date   NUMBER(16),
+    user_hard_expiration_date   NUMBER(16),
 
     PRIMARY KEY (user_uid)
 );
@@ -72,15 +74,18 @@ CREATE TABLE horde_users (
  */
 
 CREATE TABLE horde_prefs (
-    pref_uid    CHAR(255) NOT NULL,
-    pref_scope  CHAR(16) NOT NULL,
-    pref_name   CHAR(32) NOT NULL,
+    pref_uid    VARCHAR2(255) NOT NULL,
+    pref_scope  VARCHAR2(16) NOT NULL,
+    pref_name   VARCHAR2(32) NOT NULL,
 --  See above notes on CLOBs.
     pref_value  CLOB,
 
     PRIMARY KEY (pref_uid, pref_scope, pref_name)
 );
 
+CREATE INDEX pref_uid_idx ON horde_prefs (pref_uid);
+CREATE INDEX pref_scope_idx ON horde_prefs (pref_scope);
+
 
 /**
  * The DataTree tables are used for holding hierarchical data such as Groups,
@@ -96,7 +101,6 @@ CREATE TABLE horde_datatree (
     datatree_order       NUMBER(16),
     datatree_data        CLOB,
     datatree_serialized  NUMBER(8) DEFAULT 0 NOT NULL,
-    datatree_updated     DATE,
 
     PRIMARY KEY (datatree_id)
 );
@@ -135,7 +139,7 @@ CREATE TABLE horde_vfs (
     vfs_name      VARCHAR2(255) NOT NULL,
     vfs_modified  NUMBER(16) NOT NULL,
     vfs_owner     VARCHAR2(255),
-    vfs_data      CLOB,
+    vfs_data      BLOB,
 
     PRIMARY KEY   (vfs_id)
 );
@@ -144,4 +148,30 @@ CREATE INDEX vfs_path_idx ON horde_vfs (vfs_path);
 CREATE INDEX vfs_name_idx ON horde_vfs (vfs_name);
 
 
+CREATE TABLE horde_histories (
+    history_id       NUMBER(16) NOT NULL,
+    object_uid       VARCHAR2(255) NOT NULL,
+    history_action   VARCHAR2(32) NOT NULL,
+    history_ts       NUMBER(16) NOT NULL,
+    history_desc     CLOB,
+    history_who      VARCHAR2(255),
+    history_extra    CLOB,
+
+    PRIMARY KEY (history_id)
+);
+
+CREATE INDEX history_action_idx ON horde_histories (history_action);
+CREATE INDEX history_ts_idx ON horde_histories (history_ts);
+CREATE INDEX history_uid_idx ON horde_histories (object_uid);
+
+
+CREATE TABLE horde_sessionhandler (
+    session_id             VARCHAR2(32) NOT NULL,
+    session_lastmodified   INT NOT NULL,
+    session_data           BLOB,
+
+    PRIMARY KEY (session_id)
+);
+
+
 exit
