@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: horde/services/portal/edit.php,v 1.44.8.6 2007/01/02 13:55:17 jan Exp $
+ * $Horde: horde/services/portal/edit.php,v 1.44.8.7 2007/11/22 00:16:14 jan Exp $
  *
  * Copyright 1999-2007 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
@@ -29,7 +29,14 @@ if (empty($fullname)) {
 
 // Instantiate the blocks objects.
 $blocks = &Horde_Block_Collection::singleton('portal');
-$layout = &Horde_Block_Layout_Manager::singleton('portal', $blocks, $prefs->getValue('portal_layout'));
+$layout_pref = @unserialize($prefs->getValue('portal_layout'));
+if (!is_array($layout_pref)) {
+    $layout_pref = array();
+}
+if (!count($layout_pref)) {
+    $layout_pref = Horde_Block_Collection::getFixedBlocks();
+}
+$layout = &Horde_Block_Layout_Manager::singleton('portal', $blocks, $layout_pref);
 
 // Handle requested actions.
 $layout->handle(Util::getFormData('action'),
